{*******************************************************************}
{                                                                   }
{ Team AutoLink utility unit for Primary Care Management            }
{ a component of the Scheduling Redesign Project                    }
{                                                                   }
{ Scheduling ver 5.3 AUG 13, 1993                                   }      
{ Management Information Products Group                             }
{ Albany NY                                                         }
{ Dept. of Veteran Affairs                                          }
{ March 1996 (mjk)                                                  }
{                                                                   }
{ Version: 1Tx     Updated:                                         }
{                                                                   }
{*******************************************************************}

unit Autout;

interface

uses graphics, trpcb, mfunstr, classes, sysutils, scutdef;

type
  TScAutoType        = (ward, specialty, practitioner, room, clinic);


  {Basic autolink class definition}
  TScAutoLink = class(TObject)
  private
    FDisplayText : TScStr50;
    FID          : TScStr20;
    FAutoType    : TScStr20;
  public
    function Compare(link: TScAutoLink): Boolean;
    function GetLink(RPCBroker: TRPCBroker; LinkData: TStringList): Boolean;
    property DisplayText : TScStr50 read FDisplayText write FDisplayText;
    property ID          : TScStr20 read FID          write FID;
    property AutoType    : TScStr20 read FAutoType    write FAutoType;
  end;

  {Basic team class definition for autolinks.
  (May want to combine with team record definition.)}
  TScTeam = class(TObject)
  private
    FName     : TScStr50;
    FID       : TScStr20;
    FAutoLinks: TList;
    FModified : Boolean;

  public
    constructor Create;
    destructor  Destroy; override;
    procedure   SetUpLinks;
    procedure   ClearLinks;
    function    GetRecord(RecID: TScStr20; RPCBroker: TRPCBroker): Boolean;
    function    SetRecord(RPCBroker: TRPCBroker): Boolean;
    procedure   ResetRecord(RPCBroker: TRPCBroker);
    property Name      : TScStr50 read FName      write FName;
    property ID        : TScStr20 read FID        write FID;
    property AutoLinks : TList    read FAutoLinks write FAutoLinks;
  end;

{constants used throughout autolink functionality.
 Some may be useful in other areas.}
const
  u                  = '^';
  TimerOn            = True;
  TimerOff           = False;
  SC_CL_MOD          = clred;
  SC_CL_ASSIGNED     = clpurple;
  SC_CL_NOT_ASSIGNED = clblack;
  SC_FS_MOD          = [];
  SC_FS_ASSIGNED     = [];
  SC_FS_NOT_ASSIGNED = [];

  SC_FS_ONE_ONLY     = [fsBold];
  SC_FS_NONE_OK      = [fsItalic];
  SC_FS_REGULAR      = [];
  AutoType: array[TScAutoType] of TScStr20 = ('WARD', 'SPECIALTY','PRACTITIONER','ROOM','CLINIC');


implementation

{       ----------  TScAutoLink Methods  ----------}

{Allows comparing of two autolink objects for equality}
function TScAutolink.Compare(link: TScAutoLink): Boolean;
begin
  if (FAutoType = link.AutoType) and
     (FID       = link.ID)       then
       Result := True
  else
       Result := False;
end;


{Retrieves basic data from server about link that is not available
 in autolink object. Currently, it retrieves list of teams that the
 link entity is linked to.}
function TScAutoLink.GetLink(RPCBroker: TRPCBroker; LinkData: TStringList): Boolean;
var
  x: string;
  i: integer;
begin
  with RPCBroker do
    begin
      RemoteProcedure := 'SCTM AUTOLINK GETLINK';
      Param[0].PType := literal;
      Param[0].Value := FAutoType;
      Param[1].PType := literal;
      Param[1].Value := FID;
      Call;
      if Results.Count > 0 then
         begin
           GetLink := True;
           LinkData.Assign(Results);
         end
      else
         Getlink := False;
    end;

end;

{       ----------  TScTeam Methods  ----------}

{Creates the team object instance.}
constructor TScTeam.Create;
begin
  inherited Create;
  SetUpLinks;
end;
{Frees the team object instance.}
destructor TScTeam.Destroy;
begin
  ClearLinks;
  inherited destroy;
end;

{Frees link objects attached to FAutoLinks TList
 and then frees FAutoLinks itself.}
procedure TScTeam.ClearLinks;
var
   i: integer;
begin
  for i :=0 to FAutoLinks.Count - 1 do
    TScAutoLink(FAutoLinks[i]).Free;
  FAutoLinks.Free;
end;


{Creates FAutoLinks TList.}
procedure TScTeam.SetUpLinks;
begin
  FAutoLinks   := TList.Create;
end;

{Retrieves data from server for team record assoicated with RecID.}
function TScTeam.GetRecord(RecID: TScStr20; RPCBroker: TRPCBroker): Boolean;
var
  x: string;
  i: integer;
  link: TScAutoLink;
begin
  FID := RecID;
  with RPCBroker do
    begin
      RemoteProcedure := 'SCTM AUTOLINK GETRECORD';
      Param[0].PType := literal;
      Param[0].Value := FID;
      Call;
      if Results.Count > 0 then
         begin
           GetRecord := True;
           x := Results[0];
           FName := piece(x,u,1);
           for i := 1 to Results.Count - 1 do
             begin
               x := Results[i];
               link := TScAutoLink.Create;
               link.FDisplayText := piece(x,u,1);
               link.FAutoType    := piece(x,u,2);
               link.FID          := piece(x,u,3);
               FAutoLinks.Add(link);
             end;
         end
      else
         GetRecord := False;
    end;

end;

{Sends team data to server for entry/edit into database.}
function TScTeam.SetRecord(RPCBroker: TRPCBroker): Boolean;
var
   cnt,n: integer;
   link: TScAutoLink;
   ok: boolean;
begin
  cnt := 0;
  RPCBroker.RemoteProcedure := 'SCTM AUTOLINK SETRECORD';
  RPCBroker.Param[0].PType := literal;
  RPCBroker.Param[0].Value := FID;

  RPCBroker.Param[1].PType := list;
  for n := 0 to FAutoLinks.Count - 1 do
    begin
      link := TScAutoLink(FAutoLinks[n]);
      inc(cnt);
      RPCBroker.Param[1].Mult[IntToStr(cnt)] := link.DisplayText + u +
                                                link.FAutoType   + u +
                                                link.FID;
    end;

  RPCBroker.Call;
  SetRecord := (RPCBroker.Results[0] = '1');
end;

{Resets team object data from server.}
procedure TScTeam.ResetRecord(RPCBroker: TRPCBroker);
begin
  ClearLinks;
  SetUpLinks;
  GetRecord(FID,RPCBroker);
end;

end.
