{*******************************************************************}
{                                                                   }
{ Team AutoLink Selection form for Primary Care Management          }
{ a component of the Scheduling Redesign Project                    }
{                                                                   }
{ Scheduling ver 5.3 AUG 13, 1993                                   }      
{ Management Information Products Group                             }
{ Albany NY                                                         }
{ Dept. of Veteran Affairs                                          }
{ March 1996  (mjk)                                                 }
{                                                                   }
{    Last updated: 28 May 2002 (J. Peterson)                        }
{                                                                   }
{*******************************************************************}

{ Form Template - Source and Destination Choices Lists }
unit Autosel;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, Trpcb, ExtCtrls, mfunstr, Dialogs, SysUtils, Menus,
  Fmcmpnts, Fmlookup, Messages, autodet, autout, scutdef {,Gtsize32 - PDR},
  Diaccess;
type
  TfrmAutolinks = class(TForm)
    boxCandidates: TGroupBox;
    SrcList: TListBox;
    boxAutoLinks: TGroupBox;
    DstList: TListBox;
    txtSearch: TEdit;
    cmdFind: TBitBtn;
    boxInclExcl: TGroupBox;
    IncludeBtn: TSpeedButton;
    IncAllBtn: TSpeedButton;
    ExcludeBtn: TSpeedButton;
    ExAllBtn: TSpeedButton;
    chkAll: TCheckBox;
    tmr: TTimer;
    popSrc: TPopupMenu;
    cmdpopSrcInclude: TMenuItem;
    popDst: TPopupMenu;
    cmdpopDstExclude: TMenuItem;
    cmdpopDstExcludeAll: TMenuItem;
    N2: TMenuItem;
    cmdpopDstSave: TMenuItem;
    cmdpopDstCancel: TMenuItem;
    FMListerWard: TFMLister;
    FMListerSpecialty: TFMLister;
    FMListerPractitioner: TFMLister;
    FMListerRoom: TFMLister;
    FMListerClinic: TFMLister;
    pnlStatusBar: TPanel;
    Panel2: TPanel;
    lblStatus: TLabel;
    cmdpopSrcIncludeAll: TMenuItem;
    FMLookUp1: TFMLookUp;
    FMLister1: TFMLister;
    cmdSave: TBitBtn;
    cmdCancel: TBitBtn;
    cmdMore: TButton;
    lblSearch: TLabel;
    Panel3: TPanel;
    Panel1: TPanel;
    cmdpopSrcDetails: TMenuItem;
    N4: TMenuItem;
    cmdpopDstDetails: TMenuItem;
    N1: TMenuItem;
    boxRadType: TGroupBox;
    radWard: TRadioButton;
    radSpecialty: TRadioButton;
    radPractitioner: TRadioButton;
    radRoom: TRadioButton;
    radClinic: TRadioButton;
    boxLegend: TGroupBox;
    Panel4: TPanel;
    Panel6: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    GroupBox1: TGroupBox;
    cmdClose: TBitBtn;
    cmdHelp: TBitBtn;
    {GTForm1: GTForm - PDR;}
    Panel5: TPanel;
    procedure IncludeBtnClick(Sender: TObject);
    procedure ExcludeBtnClick(Sender: TObject);
    procedure IncAllBtnClick(Sender: TObject);
    procedure ExcAllBtnClick(Sender: TObject);
    procedure MoveSelected(List: TCustomListBox; Items: TStrings);
    procedure SetItem(List: TListBox; Index: Integer);
    function GetFirstSelection(List: TCustomListBox): Integer;
    procedure SetButtons;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure cmdSaveClick(Sender: TObject);
    procedure cmdCancelClick(Sender: TObject);
    procedure DstListDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure DstListDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure SrcListDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure SrcListDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure FormResize(Sender: TObject);
    procedure tmrTimer(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure cmdpopSrcDetailsClick(Sender: TObject);
    procedure cmdFindClick(Sender: TObject);
    procedure chkAllClick(Sender: TObject);
    procedure txtSearchChange(Sender: TObject);
    procedure DstListClick(Sender: TObject);
    procedure SrcListClick(Sender: TObject);
    procedure cmdMoreClick(Sender: TObject);
    procedure cmdpopDstDetailsClick(Sender: TObject);
    procedure radWardClick(Sender: TObject);
    procedure radSpecialtyClick(Sender: TObject);
    procedure radPractitionerClick(Sender: TObject);
    procedure radRoomClick(Sender: TObject);
    procedure radClinicClick(Sender: TObject);
    procedure cmdCloseClick(Sender: TObject);
  private
    { Private declarations }
    FTeamIDPassed: TScStr20;
    procedure radTypeClick;
    procedure GetData;
    procedure SetSrcItem(srcitem: string; AutoLinkText: TScStr20);
    procedure AddAutoLinks(List: TCustomListBox);
    procedure DeleteAutoLinks(List: TCustomListBox);
    procedure DeleteLink(LinkObject: TObject);
    procedure ShowLinkDetails(source: TCustomListBox);
    procedure AddLink(LinkObject: TObject);
    procedure UpdateSaveCancelBtns(btnStatus: boolean);
    procedure UpdateTypeColor;
    procedure ResetTypeColors;
    procedure ChangeDst(AutoLinkText: TScStr20);
    procedure ChangeSrc(AutoLinkText: TScStr20);
    procedure ChangeSrcX(AutoLinkText: TScStr20);
    procedure ClearListBox(list: TListBox);
    procedure UpdateCursor(Sender: TObject);
    procedure SetStatusBar(text: String ; Mode: Boolean);
  public
    { Public declarations }
    function Execute: Boolean;
    procedure FormLoad;
    property TeamIDPassed : TScStr20 read FTeamIDPassed write FTeamIDPassed;
  end;

var
  frmAutolinks : TfrmAutolinks;

implementation

uses scut2, pcmm1;

{$R *.DFM}

var
  Team             : TScTeam;
  currentLister    : TFMLister;
  currentAutoType  : TScAutoType;

procedure TfrmAutolinks.IncludeBtnClick(Sender: TObject);
var
  Index: Integer;
begin
  Index := GetFirstSelection(SrcList);
  AddAutoLinks(SrcList);
  MoveSelected(SrcList, DstList.Items);
  SetItem(SrcList, Index);
end;

procedure TfrmAutolinks.ExcludeBtnClick(Sender: TObject);
var
  Index: Integer;
begin
  Index := GetFirstSelection(DstList);
  DeleteAutoLinks(DstList);
  MoveSelected(DstList, SrcList.Items);
  SetItem(DstList, Index);
end;

procedure TfrmAutolinks.IncAllBtnClick(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to SrcList.Items.Count - 1 do
    begin
      DstList.Items.AddObject(SrcList.Items[I],SrcList.Items.Objects[I]);
      AddLink(SrcList.Items.Objects[I]);
    end;
  SrcList.Items.Clear;
  SetItem(SrcList, 0);
end;

procedure TfrmAutolinks.ExcAllBtnClick(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to DstList.Items.Count - 1 do
    begin
      SrcList.Items.AddObject(DstList.Items[I], DstList.Items.Objects[I]);
      DeleteLink(DstList.Items.Objects[I]);
    end;
  DstList.Items.Clear;
  SetItem(DstList, 0);
end;


procedure TfrmAutolinks.MoveSelected(List: TCustomListBox; Items: TStrings);
var
  I: Integer;
begin
  for I := List.Items.Count - 1 downto 0 do
    if List.Selected[I] then
    begin
      Items.AddObject(List.Items[I], List.Items.Objects[I]);
      List.Items.Delete(I);
    end;
end;

procedure TfrmAutolinks.AddAutoLinks(List: TCustomListBox);
var
  I: Integer;
  obj: TObject;
begin
  for I := List.Items.Count - 1 downto 0 do
    if List.Selected[I] then
      AddLink(List.Items.Objects[I]);
end;

{Add link object to team}
procedure TfrmAutoLinks.AddLink(LinkObject: TObject);
begin
  if Team.AutoLinks.IndexOf(LinkObject) = -1 then
    begin
      Team.AutoLinks.Add(LinkObject);
      UpdateSaveCancelBtns(True);
      UpdateTypeColor;
    end;
end;

procedure TfrmAutolinks.DeleteAutoLinks(List: TCustomListBox);
var
  I, autoi: Integer;
  obj: TObject;
begin
  for I := List.Items.Count - 1 downto 0 do
    if List.Selected[I] then
    begin
      obj := List.Items.Objects[I];
      DeleteLink(obj);
    end;
end;

{Delete link object from team}
procedure TfrmAutoLinks.DeleteLink(LinkObject: TObject);
var
  autoi: integer;
begin
  if LinkObject <> nil then
    begin
      autoi := Team.AutoLinks.IndexOf(LinkObject);
      if autoi <> -1 then
        begin
          Team.AutoLinks.Delete(autoi);
          UpdateSaveCancelBtns(True);
          UpdateTypeColor;
        end;
    end;
end;

{Set type specific panel text color to 'modify' color.}
procedure TfrmAutolinks.UpdateTypeColor;
begin
  case currentAutoType of
   ward        : radWard.Font.Color             := SC_CL_MOD;
   specialty   : radSpecialty.Font.Color        := SC_CL_MOD;
   practitioner: radPractitioner.Font.Color     := SC_CL_MOD;
   room        : radRoom.Font.Color             := SC_CL_MOD;
   clinic      : radClinic.Font.Color           := SC_CL_MOD;
  end;
end;

{set all type panels to either 'assigned' or 'not assigned' color.}
procedure TfrmAutolinks.ResetTypeColors;
var
  i    : integer;
  aType: TScAutoType;
  hit  : Boolean;
  cl   : TColor;
  fs   : TFontStyles;
  ltype, scanType : String;
begin
  for aType := ward to clinic do
    begin
      scanType := AutoType[aType];
      hit      := False;
      i        := 0;

      while (not hit) and (i <> Team.AutoLinks.Count) do
        begin
          ltype := TScAutoLink(Team.AutoLinks[i]).AutoType;
          if  ltype =  scantype then
             hit := True;
          inc(i)
        end;

      if hit then
        cl :=  SC_CL_ASSIGNED
      else
        cl :=  SC_CL_NOT_ASSIGNED;

      case aType of
        ward        : radWard.Font.Color             := cl;
        Specialty   : radSpecialty.Font.Color        := cl;
        Practitioner: radPractitioner.Font.Color     := cl;
        Room        : radRoom.Font.Color             := cl;
        Clinic      : radClinic.Font.Color           := cl;
      end;
    end;
end;

procedure TfrmAutolinks.UpdateSaveCancelBtns(btnStatus: boolean);
begin
   if cmdSave.Enabled <> btnStatus then
     begin
       cmdSave.Enabled         := btnStatus;
       cmdCancel.Enabled       := btnStatus;
       cmdpopDstSave.Enabled   := btnStatus;
       cmdpopDstCancel.Enabled := btnStatus;
     end;
end;

procedure TfrmAutolinks.SetButtons;
var
  SrcEmpty, DstEmpty, SrcSelect, DstSelect: Boolean;
begin
  SrcEmpty := SrcList.Items.Count = 0;
  DstEmpty := DstList.Items.Count = 0;

  SrcSelect := SrcList.SelCount > 0;
  DstSelect := DstList.SelCount > 0;

  IncludeBtn.Enabled          := SrcSelect;
  cmdpopSrcInclude.Enabled    := SrcSelect;
  cmdpopSrcDetails.Enabled    := SrcSelect;

  IncludeBtn.Enabled          := not SrcEmpty;
  cmdpopSrcInclude.Enabled    := not SrcEmpty;
  IncAllBtn.Enabled           := not SrcEmpty;
  cmdpopSrcIncludeAll.Enabled := not SrcEmpty;

  ExcludeBtn.Enabled          := DstSelect;
  cmdpopDstExclude.Enabled    := DstSelect;
  cmdpopDstDetails.Enabled    := DstSelect;

  ExcludeBtn.Enabled          := not DstEmpty;
  cmdpopDstExclude.Enabled    := not DstEmpty;
  ExAllBtn.Enabled            := not DstEmpty;
  cmdpopDstExcludeAll.Enabled := not DstEmpty;
end;

function TfrmAutolinks.GetFirstSelection(List: TCustomListBox): Integer;
begin
  for Result := 0 to List.Items.Count - 1 do
    if List.Selected[Result] then Exit;
  Result := LB_ERR;
end;

procedure TfrmAutolinks.SetItem(List: TListBox; Index: Integer);
var
  MaxIndex: Integer;
begin
  with List do
  begin
    SetFocus;
    MaxIndex := List.Items.Count - 1;
    if Index = LB_ERR then Index := 0
    else if Index > MaxIndex then Index := MaxIndex;
    Selected[Index] := True;
  end;
  SetButtons;
end;

{Frees link objects attached to 'list' and clears 'list'.}
procedure TfrmAutolinks.ClearListBox(list: TListBox);
var
   i  : integer;
   obj: TObject;
begin
  for i := 0 to list.Items.Count - 1 do
    begin
    obj := list.Items.Objects[i];
    if obj <> nil then
       TScAutoLink(obj).Free;
    end;
  list.Items.Clear;
end;

{Retrieve data from team object(for DstList) and server(for SrcList).}
procedure TfrmAutolinks.GetData;
var
   i: integer;
   link: TScAutoLink;
   AutoLinkText: TScStr20;
   x: string;
begin
  DstList.Items.Clear;
  AutoLinkText    := AutoType[currentAutoType];
  ChangeDst(AutoLinkText);
  ChangeSrc(AutoLinkText);
end;

{Change destination(right) list. Get data from team object.}
procedure TfrmAutolinks.ChangeDst(AutoLinkText: TScStr20);
var
   i: integer;
   link: TScAutoLink;
begin
  DstList.Items.Clear;
  for i :=0 to Team.AutoLinks.Count - 1 do
    begin
      link := TScAutoLink(Team.Autolinks[i]);
      if AutoLinkText = link.AutoType then
        dstList.Items.AddObject(link.DisplayText,link);
    end;
  SetButtons;
end;

{Change source(left) list. Get data from server.}
procedure TfrmAutolinks.ChangeSrc(AutoLinkText: TScStr20);
var
   i: integer;
   link: TScAutoLink;
   x: string;
begin
currentLister := TFMLister(FindComponent('FMLister'+ AutoLinkText));
with currentLister do
  begin
    cmdMore.Enabled := False;
    ClearListBox(srcList);
    PartList.Clear; {pdr}
    if txtSearch.Text <> '' then
      PartList.Insert(0,UpperCase(txtSearch.Text)){Part := UpperCase(txtSearch.Text) - PDR}
    else
      PartList.insert(0,'');{Part := ''; - PDR}
    SetStatusBar('Retrieving choices...', TimerOff);
    GetList(nil);

    case Results.Count of
     0: x := 'No matches found!';
     1: x := 'One match found.';
    else
        x := 'Matches found.';
    end;

    SetStatusBar(x, TimerOn);

    if FromList.count{From <> ''} <> 0 then
       cmdMore.Enabled := More;

    for i := 0 to Results.Count -1 do
      SetSrcItem(GetFMDelimRec(currentlister,i), AutoLinkText);{Results[i], AutoLinkText) - PDR}
  end;

  cmdFind.Default  := False;
  cmdClose.Default := True;
  SrcList.SetFocus;
  SetButtons;
end;

{Set source item and object}
procedure TfrmAutolinks.SetSrcItem(srcitem: string; AutoLinkText: TScStr20);
var
  link: TScAutoLink;
begin
  link             := TScAutoLink.Create;
  link.DisplayText := piece(srcItem,u,2);
  link.ID          := piece(srcItem,u,1);
  link.AutoType    := piece(AutoLinkText,u,1);
  if dstList.Items.IndexOf(link.DisplayText) = -1 then
       Srclist.Items.AddObject(link.DisplayText,link);
end;

procedure TfrmAutolinks.radTypeClick;
begin
  GetData;
  boxCandidates.Caption := AutoType[currentAutoType] + ' Choices';
  boxAutoLinks.Caption  := AutoType[currentAutoType] + ' AutoLinks';
end;

procedure TfrmAutolinks.FormCreate(Sender: TObject);
begin
  Font.Assign(FrmMain.FontDialog1.Font);
  Team := TScTeam.Create;
end;

procedure TfrmAutolinks.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Team.Free;
  ClearListBox(srcList);
  Action := caFree;
end;

procedure TfrmAutolinks.cmdSaveClick(Sender: TObject);
begin
  UpdateSaveCancelbtns(False);
  SetStatusBar('Updating record...', TimerOff);
  if Team.SetRecord(RPCBrokerMain) then
     begin
       SetStatusBar('Update successful!', TimerOn);
       ResetTypeColors;
     end;
end;

procedure TfrmAutolinks.cmdCancelClick(Sender: TObject);
begin
  SetStatusBar('Resetting AutoLink Information', TimerOff);
  UpdateSaveCancelbtns(False);
  Team.ResetRecord(RPCBrokerMain);
  ResetTypeColors;
  radTypeClick;
  SetStatusBar('Reset Complete!', TimerOn);
end;

procedure TfrmAutolinks.DstListDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
if (source is TListBox) and
   (TListBox(Source).Name = 'SrcList') then
       Accept := True;
end;

procedure TfrmAutolinks.DstListDragDrop(Sender, Source: TObject; X,
  Y: Integer);
begin
if (source is TListBox) and
   (TListBox(Source).Name = 'SrcList') then
      IncludeBtnClick(Sender);
end;

procedure TfrmAutolinks.SrcListDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
if (source is TListBox) and
   (TListBox(Source).Name = 'DstList') then
       Accept := True;
end;

procedure TfrmAutolinks.SrcListDragDrop(Sender, Source: TObject; X,
  Y: Integer);
begin
if (source is TListBox) and
   (TListBox(Source).Name = 'DstList') then
      ExcludeBtnClick(Sender);
end;

procedure TfrmAutolinks.FormResize(Sender: TObject);
begin
  Clientwidth := 500;
end;

procedure TfrmAutolinks.tmrTimer(Sender: TObject);
begin
  lblStatus.Caption := '';
  tmr.Enabled := False;
end;

procedure TfrmAutolinks.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
var
 ret: Word;
begin
cmdClose.SetFocus;
ret := mrOK;
{Has link data been modified? If so, ask user OK.}
if (radWard.Font.Color         = SC_CL_MOD) or
   (radSpecialty.Font.Color    = SC_CL_MOD) or
   (radPractitioner.Font.Color = SC_CL_MOD) or
   (radRoom.Font.Color         = SC_CL_MOD) or
   (radClinic.Font.Color       = SC_CL_MOD) then
     begin
       ret := MessageDlg('Save changes you have made to autolinks?', mtConfirmation,mbYesNoCancel , 0);
       if ret = mrYes then
         cmdSaveClick(Sender);
     end;
if ret = mrCancel then
  CanClose := False;

end;

procedure TfrmAutolinks.cmdFindClick(Sender: TObject);
begin
  changeSrc(AutoType[currentAutoType]);
end;

procedure TfrmAutolinks.chkAllClick(Sender: TObject);
begin
  if chkAll.Checked then
     begin
       txtSearch.Text        := '';
       txtSearch.Visible     := False;
       cmdFind.Visible       := False;
       lblSearch.Visible     := False;
       changeSrc(AutoType[currentAutoType]);
       cmdClose.Default      := True;
     end
  else
     begin
      txtSearch.Visible      := True;
      cmdFind.Visible        := True;
      lblSearch.Visible      := True;
      txtSearch.SetFocus;
     end;
end;

procedure TfrmAutolinks.txtSearchChange(Sender: TObject);
begin
  cmdFind.Default := True;
end;

procedure TfrmAutolinks.SrcListClick(Sender: TObject);
begin
  UpdateCursor(Sender);
  SetButtons;
end;

procedure TfrmAutolinks.DstListClick(Sender: TObject);
begin
  UpdateCursor(Sender);
  SetButtons;
end;

procedure TfrmAutoLinks.UpdateCursor(Sender: TObject);
begin
  if Sender is TListBox then
    with TListBox(Sender) do
      if SelCount > 1 then
        DragCursor := crMultiDrag
      else
        DragCursor := crDrag;
end;

procedure TfrmAutolinks.cmdMoreClick(Sender: TObject);
var
  i, start, last: integer;
  AutoLinkText: TScStr20;
begin
  cmdMore.Enabled := False;
  with currentLister do
    begin
      if More then
        begin
          start := Results.Count;
          GetMore(nil);
          cmdMore.Enabled := More;
          last  := Results.Count  - 1;
          AutoLinkText := AutoType[currentAutoType];
          for i := start to last do
            SetSrcItem(GetFMDelimRec(currentlister,i),AutoLinkText);{Results[i], AutoLinkText) - PDR}
        end;
    end;
   end;

procedure TfrmAutolinks.cmdpopDstDetailsClick(Sender: TObject);
begin
ShowLinkDetails(DstList);
end;

procedure TfrmAutolinks.cmdpopSrcDetailsClick(Sender: TObject);
begin
ShowLinkDetails(SrcList);
end;

{Retrieve and load team data for TeamIDPassed property or
 if blank, ask user to select team.}
procedure TfrmAutolinks.FormLoad;
var
  newrec, loaded :Boolean;
begin
  newrec := False;
  loaded := False;

  if TeamIDPassed = '' then
    begin
      SetStatusBar('Activating Team Lookup Form...', TimerOff);
      if FMLookup1.Execute(newrec) then
        TeamIDPassed := FMLookup1.RecordNumber;
    end;

  if TeamIDPassed <> '' then
    begin
      SetStatusBar('Retrieving Team AutoLinks...', TimerOff);
      if Team.GetRecord(TeamIDPassed, RPCBrokerMain) then
        begin
          Caption           := 'AutoLink Specification (Team: ' + Team.Name + ')';
          chkAll.Checked    := False;
          txtSearch.Text    := 'A';
          radWard.Checked   := True;
          loaded            := True;
          ResetTypeColors;
          SetStatusBar('AutoLinks Loaded!', TimerOn);
        end;
    end;

  if not loaded then
    PostMessage(Handle, WM_CLOSE, 0, 0);
end;

{Display to user link entity information.}
procedure TfrmAutolinks.ShowLinkDetails(source: TCustomListBox);
var
 lst: TStringList;
 obj: TScAutoLink;
begin
 if SrcList.ItemIndex <> -1 then
   begin
     obj := TScAutoLink(source.Items.Objects[source.ItemIndex]);
     lst := TStringList.Create;
     try
       if obj.GetLink(RPCBrokerMain, lst) then
         begin
           Application.CreateForm(TfrmAutoDetail, frmAutoDetail);
           with frmAutoDetail do
             begin
               lblNameText.Caption := obj.DisplayText;
               lblIDText.Caption   := obj.ID;
               lblTypeText.Caption := obj.AutoType;
               lstAutoLinks.Items.Assign(lst);
               ShowModal;
               Free;
             end;
         end;
     finally
       lst.Free;
     end;
   end;
end;

{Updates status bar text and turns on/off timer.
 Turning on timer will cause text to be cleared
 automatically.}
procedure TfrmAutoLinks.SetStatusBar(text: String ; Mode: Boolean);
begin
  tmr.Enabled       := TimerOff;
  lblStatus.Caption := ' ' + text;
  tmr.Enabled       := mode;
  Application.ProcessMessages;
end;

{NOT USED: Kept for reference purposes.}
function TfrmAutoLinks.Execute: Boolean;
begin
  Result := False ;
  try
     frmAutoLinks := TfrmAutoLinks.Create(Application);
     {frmAutoLinks.Show;}
     result := true;
  except
     on E: Exception do
        messagedlg(E.Classname, mtWarning, [mbOk], 0);
  end;
end;

{NOT USED: Kept for reference purposes.}
procedure TfrmAutolinks.ChangeSrcX(AutoLinkText: TScStr20);
var
   i: integer;
   link: TScAutoLink;
   x: string;
begin
with RPCBrokerMain do
  begin
    ClearListBox(srcList);
    RemoteProcedure := 'SCTM AUTOLINK GETALL';
    Param[0].PType := literal;
    Param[0].Value := AutoLinkText;
    Call;
    for i := 0 to Results.Count -1 do
      begin
        x                := Results[i];
        link             := TScAutoLink.Create;
        link.DisplayText := piece(x,u,1);
        link.ID          := piece(x,u,3);
        link.AutoType    := piece(x,u,2);
        if dstList.Items.IndexOf(link.DisplayText) = -1 then
             Srclist.Items.AddObject(link.DisplayText,link);
      end;
  end;
end;

procedure TfrmAutolinks.radWardClick(Sender: TObject);
begin
  currentAutoType := ward;
  radTypeClick;
end;

procedure TfrmAutolinks.radSpecialtyClick(Sender: TObject);
begin
  currentAutoType := specialty;
  radTypeClick;

end;

procedure TfrmAutolinks.radPractitionerClick(Sender: TObject);
begin
  currentAutoType := practitioner;
  radTypeClick;

end;

procedure TfrmAutolinks.radRoomClick(Sender: TObject);
begin
  currentAutoType := room;
  radTypeClick;

end;

procedure TfrmAutolinks.radClinicClick(Sender: TObject);
begin
  currentAutoType := clinic;
  radTypeClick;

end;

procedure TfrmAutolinks.cmdCloseClick(Sender: TObject);
begin
 close;
end;

end.
