{*******************************************************************}
{                                                                   }
{    Generic form for the Multiple Team Assignment/Discharge        }
{    and Multiple Position Assignments / discharges                 }
{    for Primary Care Management.                                   }
{                                                                   }
{    Scheduling ver 5.3 AUG 13, 1993                                 }      
{    Management Information Products Group                          }
{    Albany NY (SCK)                                                }
{    Dept. of Veteran Affairs                                       }
{                                                                   }
{    Last updated: 28 May 2002 (J. Peterson)                        }
{                                                                   }
{*******************************************************************}

unit Mtassign;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, Trpcb, Fmlookup, Fmcmpnts, Pcmm1, Scut3, Scut2, Scutbrk,
  Mfunstr, SysUtils, Messages, Dialogs, ExtCtrls, Fmcntrls,
  Scutdef, OvcSc, OvcBase, OvcEF, OvcPB, OvcPF, OvcDT, Menus, Printers
  {,Gtsize32, GTVASzr- PDR}, Diaccess;

const
  DEBUG  = false;
  NUM_RETURN = 200;
  CLINIC_SCREEN = 'IF $P($G(^(0)),U,3)="C"';

{ basic patient id record}
type
  TPatientID = class(TObject)
  private
    FName       : TScStr50;
    FDfn        : TScStr20;
    FENrollDT   : TScStr20;
  public
    property Name : TScStr50 read FName  write FName;
    property DFN  : TScStr20 read FDfn   write FDfn;
    property EDate: TScStr20 read FEnrollDT write FEnrollDT;
  end;

  TfrmMltAssign = class(TForm)
    cmdSave: TBitBtn;
    HelpBtn: TBitBtn;
    SrcList: TListBox;
    DstList: TListBox;
    SrcLabel: TLabel;
    DstLabel: TLabel;
    IncludeBtn: TSpeedButton;
    IncAllBtn: TSpeedButton;
    ExcludeBtn: TSpeedButton;
    ExAllBtn: TSpeedButton;
    ltrTeams: TFMLister;
    FMLookUp1: TFMLookUp;
    grpSelectFrame: TGroupBox;
    cboSource: TFMComboBoxLookUp;
    lblSelectFrom: TLabel;
    cmdMore: TButton;
    LtrClinics: TFMLister;
    cmdSelectEntry: TButton;
    cmdClose: TBitBtn;
    Bevel1: TBevel;
    Bevel2: TBevel;
    cmdCancel: TBitBtn;
    Label1: TLabel;
    lblCount: TLabel;
    lbljob: TLabel;
    Label2: TLabel;
    lblProcessed: TLabel;
    Bevel4: TBevel;
    grpOptions1: TGroupBox;
    chkType: TCheckBox;
    chkRestrict: TCheckBox;
    OvcController1: TOvcController;
    Bevel3: TBevel;
    lstPositionSet1: TListBox;
    lstPositionSet2: TListBox;
    cboSelectBy: TComboBox;
    Label3: TLabel;
    Bevel5: TBevel;
    ltrStopCodes: TFMLister;
    ltrPositions: TFMLister;
    txtFrom: TEdit;
    txtTo: TEdit;
    Label4: TLabel;
    Label5: TLabel;
    cmdSelectAll: TButton;
    FMLookUp2: TFMLookUp;
    Bevel6: TBevel;
    getTeamInfo: TFMGets;
    getPosAsgnInfo: TFMGets;
    PopupMenu1: TPopupMenu;
    Print1: TMenuItem;
    PrintDialog1: TPrintDialog;
    PopupMenu2: TPopupMenu;
    Print2: TMenuItem;
    {GTVAForm1: GTVAForm - PDR;}
    procedure IncludeBtnClick(Sender: TObject);
    procedure ExcludeBtnClick(Sender: TObject);
    procedure IncAllBtnClick(Sender: TObject);
    procedure ExcAllBtnClick(Sender: TObject);
    procedure MoveSelected(List: TCustomListBox; Items: TStrings);
    procedure SetItem(List: TListBox; Index: Integer);
    function GetFirstSelection(List: TCustomListBox): Integer;
    procedure SetButtons;
    procedure CancelBtnClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure cboSelectClick(Sender: TObject);
    procedure cboSourceExit(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormResize(Sender: TObject);
    procedure cmdSelectEntryClick(Sender: TObject);
    procedure cmdCloseClick(Sender: TObject);
    procedure cmdSaveClick(Sender: TObject);
    procedure cmdMoreClick(Sender: TObject);
    procedure cmdCancelClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure SrcListDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure DstListDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure DstListDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure SrcListDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure SrcListClick(Sender: TObject);
    procedure DstListClick(Sender: TObject);
    procedure cmdSelectAllClick(Sender: TObject);
    procedure cboSourceChange(Sender: TObject);
    procedure PCPatNoTeam(Sender: TObject);
    procedure Print1Click(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure SrcListKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure DstListKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
  private
    { Private declarations }
     PatJobID, Job : string[30];
     PatJobCount, BlockCount, LastP, StartP, ListCount : integer;
     pList : TStrings;
     procedure LoadPatientBlock(s : string; SBlock, EBlock: integer);
     procedure GetData;
     procedure SetSrcItem(s : string);
     procedure UpdateSaveCancelBtns(status : boolean);
     procedure BuildAssignments( sendlist : TStrings);
     procedure FileAssignments( jobID : string);
     procedure FileAllAssignments;
     procedure FilePosAssignments( jobID : string);
     procedure FileAllPosAssignments;
     procedure ClearListBox(list : TListBox);
     {procedure LoadTeamMembers;}
     {procedure FileDischarges( jobID : string);}
     procedure SelectPosition;
     procedure SelectTeam;
     procedure UpdateCursor(Sender: TObject);
     procedure SetDateRange(var DtBegin, DtEnd : TScStr20);
     procedure GetDateValue(var Dt : TScStr20);
     function GetTeam: string;
     function ConfirmTeamSave(cnt : integer) : boolean;
     function ConfirmPosSave(cnt : integer) : boolean;
     function TodayStr: String;
  public
    { Public declarations }
    FormMode  : TMultFormMode;
    TeamID    : TAssignmentData;
    id : TPatientID;
    procedure AssignTeam;
    procedure AssignToPosition;
    {procedure DischargeTeam;}
  end;

var
  frmMltAssign   : TfrmMltAssign;
  CurrentPat     : integer;

implementation

uses
  Enterdt, Dtrange, Plswait, dmuTeam, dmuTeamPosition;

{$R *.DFM}


{=======================================================}
{      Form actions                                     }
{=======================================================}

procedure TfrmMltAssign.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  ClearListBox(srcList);
  ClearListBox(dstList);
  {TeamID.free;}
  Action := caFree;
end;

procedure TfrmMltAssign.FormCreate(Sender: TObject);
begin
  Self.top := 0;
  Self.left := 0;
  TeamID := TAssignmentData.Create;
  cboSelectClick(nil);
  UpdateSaveCancelBtns(False);
end;

procedure TfrmMltAssign.FormShow(Sender: TObject);
begin
{  if grpSelectFrame.visible then
     cboSource.Setfocus;   }
end;

procedure TfrmMltAssign.FormKeyPress(Sender: TObject; var Key: Char);
begin
  Key := UpCase(Key);
end;

procedure TfrmMltAssign.FormResize(Sender: TObject);
begin
  ClientHeight := 360 ;
  ClientWidth := 534;
  Left := 0;
  top := 0;
end;

procedure TfrmMltAssign.cmdCloseClick(Sender: TObject);
begin
  close;
end;


{=======================================================}
{      Selection List Processing                        }
{=======================================================}

procedure TfrmMltAssign.IncludeBtnClick(Sender: TObject);
var
  Index: Integer;
begin
  Index := GetFirstSelection(SrcList);
  MoveSelected(SrcList, DstList.Items);
  SetItem(SrcList, Index);
  UpdateSaveCancelBtns(True);
end;

procedure TfrmMltAssign.ExcludeBtnClick(Sender: TObject);
var
  Index: Integer;
begin
  Index := GetFirstSelection(DstList);
  MoveSelected(DstList, SrcList.Items);
  SetItem(DstList, Index);
  UpdateSaveCancelBtns((DstList.items.count > 0))
end;

procedure TfrmMltAssign.IncAllBtnClick(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to SrcList.Items.Count - 1 do
    DstList.Items.AddObject(SrcList.Items[I],
      SrcList.Items.Objects[I]);
  SrcList.Items.Clear;
  SetItem(SrcList, 0);
  UpdateSaveCancelBtns(true);
end;

procedure TfrmMltAssign.ExcAllBtnClick(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to DstList.Items.Count - 1 do
    SrcList.Items.AddObject(DstList.Items[I], DstList.Items.Objects[I]);
  DstList.Items.Clear;
  SetItem(DstList, 0);
  UpdateSaveCancelBtns(False);
end;

procedure TfrmMltAssign.MoveSelected(List: TCustomListBox; Items: TStrings);
var
  I: Integer;
begin
  for I := List.Items.Count - 1 downto 0 do
    if List.Selected[I] then
    begin
      Items.AddObject(List.Items[I], List.Items.Objects[I]);
      List.Items.Delete(I);
    end;
end;


function TfrmMltAssign.GetFirstSelection(List: TCustomListBox): Integer;
begin
  for Result := 0 to List.Items.Count - 1 do
    if List.Selected[Result] then Exit;
  Result := LB_ERR;
end;

procedure TfrmMltAssign.SetItem(List: TListBox; Index: Integer);
var
  MaxIndex: Integer;
begin
  with List do
  begin
    SetFocus;
    MaxIndex := List.Items.Count - 1;
    if Index = LB_ERR then Index := 0
    else if Index > MaxIndex then Index := MaxIndex;
    Selected[Index] := True;
  end;
  SetButtons;
end;

procedure TfrmMltAssign.SetSrcItem(s : string);
//var
//  id : TPatientID;
begin
  id := TPatientID.Create;
  id.Name  := Piece(s, U, 2);
  id.DFN   := Piece(s, U, 1);
  id.EDate := Piece(s, U, 4); {pdr - erroneously set to piece 3 in original}
  if dstList.Items.IndexOf(id.Name) = -1 then
     srcList.Items.AddObject(id.Name, id);
end;


{================================================================}
{            Drag and Drop functionality                         }
{================================================================}

procedure TfrmMltAssign.SrcListDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  if (source is TListBox) and
     (TListBox(Source).Name = 'DstList') then
        Accept := True;
end;

procedure TfrmMltAssign.DstListDragDrop(Sender, Source: TObject; X,
  Y: Integer);
begin
  if (source is TListBox) and
     (TListBox(Source).Name = 'SrcList') then
        IncludeBtnClick(Sender);
end;

procedure TfrmMltAssign.DstListDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  if (source is TListBox) and
     (TListBox(Source).Name = 'SrcList') then
        Accept := True;
end;

procedure TfrmMltAssign.SrcListDragDrop(Sender, Source: TObject; X,
  Y: Integer);
begin
  if (source is TListBox) and
     (TListBox(Source).Name = 'DstList') then
        ExcludeBtnClick(Sender);
end;

procedure TfrmMltAssign.SrcListClick(Sender: TObject);
begin
  UpdateCursor(Sender);
  SetButtons;
end;

procedure TfrmMltAssign.DstListClick(Sender: TObject);
begin
  UpdateCursor(Sender);
  SetButtons   ;
end;

procedure TfrmMltAssign.UpdateCursor(Sender: TObject);
begin
  if Sender is TListBox then
     with TListBox(Sender) do
        if SelCount > 1 then
           DragCursor := crMultiDrag
        else
           DragCursor := crDrag;
end;

procedure TfrmMltAssign.SetButtons;
var
  SrcEmpty, DstEmpty, SrcSelect, DstSelect : Boolean;
begin
  SrcEmpty := SrcList.Items.Count = 0;
  DstEmpty := DstList.Items.Count = 0;

  SrcSelect := SrcList.SelCount > 0;
  DstSelect := DstList.SelCount > 0;

  IncludeBtn.ENabled := SrcSelect;
  IncludeBtn.ENabled := not SrcEmpty;
  IncAllBtn.ENabled := not SrcEmpty;

  ExcludeBtn.ENabled := DstSelect;
  ExcludeBtn.ENabled := not DstEmpty;
  ExAllBtn.ENabled := not DstEmpty;
end;



{=======================================================}
{      General Form Procedures                          }
{=======================================================}

procedure TfrmMltAssign.SetDateRange(var DtBegin, DtEnd : TScStr20);
begin
  Application.CreateForm(TfrmDateRange, frmDateRange);
  if frmDateRange.ShowModal = mrOk then
    begin
     DtBegin := StrToFMDate(frmDateRange.txtDtBegin.AsString);
     DtEnd := StrToFmDate(frmDateRange.txtDtEnd.AsString);
    end
  else
    begin
     DtBegin := '-1';
     DtEnd := '-1';
    end;
end;

procedure TfrmMltAssign.GetDateValue(var Dt : TScStr20);
begin
  Application.CreateForm(TfrmDateRange, frmDateRange);
  frmDateRange.txtDtEnd.visible := false;
  frmDateRange.OvcSpinner2.visible := false;
  frmDateRange.label2.visible := false;
  frmDateRange.label1.Caption := 'Effective Date:';
  if frmDateRange.ShowModal = mrOk then begin
     Dt := StrToFMDate(frmDateRange.txtDtBegin.AsString);
     end
  else begin
     Dt := '-1';
  end;
end;


procedure TfrmMltAssign.cboSourceChange(Sender: TObject);
begin
  cmdSelectEntry.Enabled := (cboSource.ItemIndex > -1);
  cmdSelectAll.Enabled := ((cboSource.ItemIndex > -1) and (SrcList.Items.Count = 0));
end;

procedure TfrmMltAssign.cmdSelectAllClick(Sender: TObject);
var
  s : string;
begin
  case cboSelectBy.ItemIndex of
     0  : s := ' enrolled in this clinic ';
     1  : s := ' associated with this Stop Code, for the selected date range ';
     2  : s := ' with an appointment in the selected date range ';
     3  : s := ' currently enrolled in this team ';
     4  : s := ' PC assignment but no PC Team ';
     5  : s := ' currently assigned to this position ';
  end;

  if messagedlg('If you click OK, all patients' + s + 'will be selected.' +
                 '  Do you want to continue?', mtConfirmation,
                 [mbYes, mbCancel], 0) = mrYes then begin
     case FormMode of
        AssignTm       : FileAllAssignments;
        AssignPos      : FileAllPosAssignments;
     end;
  end;
end;


procedure TfrmMltAssign.CancelBtnClick(Sender: TObject);
begin
  Close;
end;


procedure TfrmMltAssign.cmdMoreClick(Sender: TObject);
begin
  if DstList.Items.Count > 0 then begin
     MessageDlg('Please save currently selected entries' + RETURN +
                'before selecting the next block of patients.', mtWarning, [mbOK], 0);
     end
  else begin
     ClearListBox(srcList);
     StartP := StartP + NUM_RETURN;
     LastP := LastP + NUM_RETURN;
     case cboSelectBy.ItemIndex of
        0  : LoadPatientBlock('SCCLPT', StartP, LastP);
        1  : LoadPatientBlock('SCSCDE', StartP, LastP);
        2  : LoadPatientBlock('SCAPP', StartP, LastP);
        3  : LoadPatientBlock('SCTEAM', StartP, LastP);
        4  : LoadPatientBlock('SCPCNO', StartP, LastP);
        5  : LoadPatientBlock('SCPOS', StartP, LastP);
     end;
     SetButtons; {check...}
  end;
end;

procedure TfrmMltAssign.LoadPatientBlock(s : string; SBlock, EBlock: integer);
var
  i  : integer;
begin
  RPCBrokerMain.remoteProcedure := 'SC GET PAT BLOCK';
  with RPCBrokerMain.Param[0] do
     begin
        Value := '.SC';
        Ptype := list;

        Mult['"JOBID"']   := s;
        Mult['"JOB"']     := PatJobID;
        Mult['"BSTART"']  := IntToStr(SBlock);
        Mult['"BEND"']    := IntToStr(EBlock);
        Mult['"BLAST"']   := IntToStr(PatJobCount);
     end;
  RPCBrokerMain.Call;

  for i := 0 to RPCBrokerMain.Results.Count - 1 do
     SetSrcItem(RPCBrokerMain.Results[i]);
  cmdMore.ENabled := (SrcList.Items.Count < PatJobCount)
end;


procedure TfrmMltAssign.UpdateSaveCancelBtns(status : boolean);
begin
  if cmdSave.Enabled <> status then begin
     cmdSave.Enabled := status;
     cmdCancel.enabled := status;
  end;
end;


procedure TfrmMltAssign.cboSelectClick(Sender: TObject);
var
  team : string;
begin
  cboSource.Clear;
  ClearListBox(srcList);
  ClearListBox(dstList);
  if not cboSource.Enabled then begin
     cboSource.enabled := true;
     cmdSelectEntry.Enabled := true;
     cmdSelectEntry.OnClick := cmdSelectEntryClick;
  end;

  case cboSelectBy.ItemIndex of
     0: begin
           lblSelectFrom.Caption := 'Select Clinic:';
           with cboSource do begin
              FMLister := ltrClinics;
              FMFile := FMLIster.FileNumber;
              FMLister.Screen := CLINIC_SCREEN;
           end;
        end;

     1: begin
           lblSelectFrom.Caption := 'Select Stop Code:';
           with cboSource do begin
              FMLister := ltrStopCodes;
              FMFile := FMLIster.FileNumber;
           end;
        end;

     2: begin
           lblSelectFrom.Caption := 'Select by Clinic Appointment:';
           with cboSource do begin
              FMLister := ltrClinics;
              FMFile := FMLIster.FileNumber;
              FMLister.Screen := CLINIC_SCREEN;
           end;
        end;

     3: begin
           lblSelectFrom.Caption := 'Select Team:';
           with cboSource do begin
              FMLister := ltrTeams;
              FMFile := FMLIster.FileNumber;
           end;
        end;

     4: begin
           lblSelectFrom.Caption := 'PC Patients with no PC Team';
           cboSource.enabled := false;
           cmdSelectEntry.Enabled := true;
           cmdSelectEntry.OnCLick := PCPatNoTeam;
        end;

     5: begin
           team := GetTeam;
           if team = emptystr then
              begin
                 frmMain.StatusMsg('No team selected', true);
                 cboSelectBy.ItemIndex := 0;
              end
           else begin
              lblSelectFrom.Caption := 'Select Position:';
              with cboSource do begin
                 FMLister := ltrPositions;
                 FMLister.Screen := 'IF $P(^(0),U,2)="' + Piece(team, U, 1) +
                                     '",+$$ACTTP^SCMCTPU(Y)=1';
                 {FMLister.Part := Piece(team, U, 1);}
                 FMFile := FMLIster.FileNumber;
              end;
           end;
        end;
   end;
end;

function TfrmMltAssign.GetTeam: string;
var
  new : boolean;
begin
  if FMLookup2.execute(New) then
     result := GetFMDelimRec(FMLookup2.FMLister,FMLookup2.RecordIndex){FMLookup2.FMLister.Results[FMLookup2.RecordIndex] - PDR}
  else
     result := emptystr;
end;

procedure TfrmMltAssign.cboSourceExit(Sender: TObject);
begin
  if (ActiveControl = cmdClose) then exit;

  if not cmdSelectEntry.Enabled then begin
     cboSourceChange(nil);
     if cmdSelectEntry.enabled then
        cmdSelectentry.Setfocus;
  end;

  if cboSource.FMModified then exit;

	with cboSource do begin
     if ItemIndex >= 0 then begin
        FMctrlInternal{FMInternal - PDR} := Piece(GetFMDelimRec(FMLister,ItemIndex), '^', 1);{Piece(FMLister.Results[ItemIndex], '^', 1); - PDR}
     end;
  end;
end;

procedure TfrmMltAssign.cmdSelectEntryClick(Sender: TObject);

var

  s: string;
  DtBegin, DtEnd : TScStr20;
  Long: Boolean;

begin

  Long := false;
  cmdSelectAll.Enabled := false;
  lblCount.caption := '0';
  lblProcessed.caption := '0';
  DtBegin := emptystr;
  DtEnd := emptyStr;

  if not (cboSource.ItemIndex > -1) then
     exit;

  with RPCBrokerMain do
  begin
    {S := Piece(cboSource.FMLister.Results[cboSource.ItemIndex], U, 1); - PDR}
    S := Piece(GetFMDelimRec(cboSource.FMLister,cboSource.ItemIndex), U, 1);
    case cboSelectBy.ItemIndex of
     0: begin
          RemoteProcedure := 'SC BLD PAT CLN LIST';
          Param[0].PType := list;
          Param[0].Mult['"CLINIC"'] := S;
        end;
     1: begin
          SetDateRange(DtBegin, DtEnd);
          if (DtBegin = '-1') or (DtEnd = '-1') then exit;
          RemoteProcedure := 'SC BLD PAT SCDE LIST';
          Param[0].PType := list;
          Param[0].Mult['"STOPCODE"'] := S;
          Param[0].Mult['"MORE"'] := '0';
        end;
     2: begin
           SetDateRange(DtBegin, DtEnd);
           if (DtBegin = '-1') or (DtEnd = '-1') then exit;
           RemoteProcedure := 'SC BLD PAT APT LIST';
           Param[0].PType := list;
           Param[0].Mult['"CLINIC"'] := S;
           Param[0].Mult['"MORE"']   := '0';
        end;
     3: begin
          dmcTeam.CreateTeamRecord(S);
          with Team do
          begin
            if CurrentAssigned > 500 then
              Long := True;
            if ErrCode = 10004 then
              Long := True;
          end;
          RemoteProcedure := 'SC BLD PAT TM LIST';
          Param[0].PType := list;
          Param[0].Mult['"FROMTEAM"'] := S;
        end;
     {4: RPCBrokerMain.remoteProcedure := 'SC BLD NOPC TM LIST';}
     5: begin
          dmcPosition.CreatePositionRecord(s);
          with TeamPosition do
          begin
            if CurrentAssigned > 500 then
              Long := true;
            if ErrCode = 10004 then
              Long := True;
          end;
          RemoteProcedure := 'SC BLD PAT POS LIST';
          Param[0].PType := list;
          Param[0].Mult['"FROMPOS"'] := S;
        end;
    end;
   with Param[0] do
     begin
       Mult['"TEAM"'] := TeamID.TeamIen;
       if DtBegin <> emptyStr then
         Mult['"BEGIN"'] := DtBegin
       else
         Mult['"BEGIN"'] := TodayStr;

       if DtEnd <> emptyStr then
         Mult['"END"'] := DtEnd
       else
         Mult['"END"'] := TodayStr;

       if TeamID.PositionIen <> emptyStr then
         Mult['"POSITION"'] := TeamID.PositionIen;
     end;

   if Long then
   begin
     //frmMain.StatusSplash(1,true,true);
     //bp/cmf 204t0 RPCTimeLimit := 300;
     Call;
     //frmMain.StatusSplash(0,false,false);
     //bp/cmf 204t0 RPCTimeLimit := 30;
     end
   else Call;

   PatJobID := Piece(Results[0], U, 1);
   s := Piece(Results[0], U, 2);

  end;

  if not (s = emptyStr) then
     PatJobCount := StrToInt(s)
  else
     PatJobCount := 0;

  lblCount.Caption := IntToStr(PatJobCount);

  if PatJobCount = 0 then begin
     messagedlg('There are no patients who' + RETURN +
                'can be assigned to this team', mtInformation, [mbOk], 0);
     SrcList.Clear ;
     end
  else
     GetData;

  SetButtons ;
end;


procedure TfrmMltAssign.PCPatNoTeam(Sender: TObject);
var
  Dt : TScStr20;
  S  : string;
begin
  cmdSelectAll.Enabled := false;
  lblCount.caption := '0';
  lblProcessed.caption := '0';

  RPCBrokerMain.remoteProcedure := 'SC BLD NOPC TM LIST';
  GetDateValue(Dt);

  with RPCBrokerMain.Param[0] do
     begin
        Value := '.SC';
        Ptype := list;

        Mult['"DATE"'] := Dt;
        Mult['"TEAM"']   := TeamID.TeamIen;

        If TeamID.PositionIen <> emptyStr then
           Mult['"POSITION"'] := TeamID.PositionIen;
     end;

  RPCBrokerMain.Call;

  PatJobID := Piece(RPCBrokerMain.Results[0], U, 1);
  s := Piece(RPCBrokerMain.Results[0], U, 2);

  if not (s = emptyStr) then
     PatJobCount := StrToInt(s)
  else
     PatJobCount := 0;

  lblCount.Caption := IntToStr(PatJobCount);

  if PatJobCount = 0 then begin
     messagedlg('There are no patients who' + RETURN +
                'can be assigned to this team', mtInformation, [mbOk], 0);
     SrcList.Clear ;
     end
  else
     GetData;

end;

procedure TfrmMltAssign.GetData;
begin
  ClearListBox(srcList);
  ClearListBox(dstList);
  StartP := 1;
  LastP := NUM_RETURN;

  case cboSelectBy.ItemIndex of
     0  : LoadPatientBlock('SCCLPT', StartP, LastP);
     1  : LoadPatientBlock('SCSCDE', StartP, LastP);
     2  : LoadPatientBlock('SCAPP', StartP, LastP);
     3  : LoadPatientBlock('SCTEAM', StartP, LastP);
     4  : LoadPatientBlock('SCPCNO', StartP, LastP);
     5  : LoadPatientBlock('SCPOS', StartP, LastP);
  end;
end;

procedure TfrmMltAssign.cmdSaveClick(Sender: TObject);
var
  i     : integer;
  obj   : TObject;
  AList : TStrings;
  s     : string[20];
begin
  AList := TStringList.Create;
  try
     UpdateSaveCancelBtns(False);
     for i := DstList.Items.Count - 1 downto 0 do begin
        obj := DstList.Items.Objects[i];
        s := TPatientID(obj).DFN;
        AList.Add(s);
        DstList.Items.Delete(i);
        end;
  finally
     BuildAssignments(AList);
     lblProcessed.Caption := IntToStr(AList.count);
     case FormMode of
        AssignTm       : FileAssignments(job);

        DischargeTm    : NOTYET;

        AssignPos      : FilePosAssignments(job);

        DischargePos   : notyet;
     end;
     AList.Free;
     SetButtons ;
  end;
end;

procedure TfrmMltAssign.cmdCancelClick(Sender: TObject);
begin
  ClearListBox(srcList);
  ClearListBox(dstList);
  UpdateSaveCancelBtns(False);
  SetButtons ;
  {broker call to clear ^TMP global}
end;

procedure TfrmMltAssign.BuildAssignments( sendlist : TStrings);
var
  i  : integer;
begin
  RPCBrokerMain.remoteProcedure := 'SC BLD PAT LIST';
  with RPCBrokerMain.Param[0] do
     begin
        Value  := '"[Start]"';
        Ptype  := literal;
     end;
  RPCBrokerMain.Call;

  if RPCBrokerMain.Results[0] = '0' then begin
     messagedlg('Unable to build patient list on server', mtWarning, [mbOk], 0);
     exit;
  end
  else
     job := RPCBrokerMain.Results[0];

  if debug then begin
     lbljob.caption := job;
     lbljob.visible := true;
     end;

  FrmMain.StatusMsg('Building Patient List...', false);

  for i := 0 to sendlist.count - 1 do begin
     RPCBrokerMain.remoteProcedure := 'SC BLD PAT LIST';
     with RPCBrokerMain.Param[0] do
        begin
           Value  := job + '^' + Piece(SendList[i], U, 1);
           Ptype  := literal;
        end;
     RPCBrokerMain.Call;

     if (RPCBrokerMain.Results[0] = '0') then begin
        messagedlg('Could not continue processing patients', mterror, [mbOk], 0);
        exit;
     end;
  end;
end;


procedure TfrmMltAssign.ClearListBox(list : TListBox);
var
  i     : integer;
  obj   : TObject;
begin
  for i := 0 to list.Items.Count - 1 do begin
     obj := list.Items.Objects[i];
     if Obj <> nil then
        TPatientID(obj).free;
  end;
  list.Items.Clear;
end;

procedure AddPatients(List : TCustomListBox);
var
  i   : Integer;
  obj : TObject;
begin


end;

{=======================================================}
{      Multiple Team Assignment Procedures              }
{=======================================================}

procedure TfrmMltAssign.AssignTeam;
begin
  DstLabel.Caption := 'New Assignments';
  SrcLabel.Caption := 'Available to Assign';
  //grpOptions1.Visible := true;  patch 177
  cboSelectBy.Items.Addstrings(lstPositionSet1.Items);
  cboSelectBy.ItemIndex := 0;
  cboSelectClick(nil);
  grpSelectFrame.Visible := true;
  SelectTeam;
  Self.Caption := 'Multiple Patient Assignments to Team: ' +  TeamID.TeamName;
end;

procedure TfrmMltAssign.SelectTeam;
var
  new :  boolean;
begin
  if TeamID.TeamIen = '' then
     close;
end;



procedure TfrmMltAssign.FileAssignments( jobID : string);
var
  i1, i2 : integer;
  s1, s2 : string[10];
  ErrLog : TScError;
  x      : string;
begin
  if not ConfirmTeamSave(DstList.Items.Count) then exit;
  frmWait.lblWait.Caption := 'Please wait, processing your selections';
  frmWait.Show;
  application.ProcessMessages;
  RPCBrokerMain.remoteProcedure := 'SC FILE PAT TM ASGN';
  //bp/cmf 177 remove//RPCBrokerMain.RPCVersion := '1';
  with RPCBrokerMain.Param[0] do
     begin
        Value := '.SC';
        Ptype := list;

        Mult['"JOB"']  := jobID;
        Mult['"TEAM"'] := TeamID.TeamIen;
        Mult['"DATE"'] := TodayStr;

        if chkType.Checked then
           Mult['"TYPE"'] := '1'
        else
           Mult['"TYPE"'] := '99';

        if chkRestrict.Checked then
           Mult['"RESTRICT"'] := '1'
        else
           Mult['"RESTRICT"'] := '0';

     end;

  RPCBrokerMain.Call;
  frmWait.close;
  if RPCBrokerMain.Results[0] = '0' then
   begin
    MessageDlg('Filer NOT queued on server!',mtWarning,[mbOK],0);
    lblProcessed.Caption := '0';
   end
  else
   frmMain.StatusMsg('Patient assignments filed...',true);

  { x := RPCBrokerMain.Results[1];
  RPCBrokerMain.Results.Delete(1);
  lblProcessed.Caption := Piece(x, U, 1);

  if Piece(RPCBrokerMain.Results[0], U, 1) <> '1' then
     begin
        errlog := TScError.Create;
        errlog.LoadandDisplay(RPCBrokerMain.Results);
        errlog.free;
     end;

  if Piece(x, U, 1) <> emptystr then
     if Piece(x, U, 1) <> '0' then
        frmMain.StatusMsg('Patient assignments filed...', true);}

end;

procedure TfrmMltAssign.FileAllAssignments;
var
  ErrLog : TScError;
  x, source      : string;
  DtBegin, DtEnd : TScStr20;
begin
  DtBegin := emptystr;    DtEnd := emptyStr;

  RPCBrokerMain.remoteProcedure := 'SC FILE ALL PAT TM ASGN';
  with RPCBrokerMain.Param[0] do
     begin
        Value := '.SC';
        Ptype := list;

        case cboSelectBy.ItemIndex of
           0  :  begin
                    source := 'CLN^';
                    frmWait.lblWait.Caption :=
                        'Processing clinics, this could take a while.  Please wait.'
                 end;

           1  :  begin
                    source := 'STOPC^';
                    SetDateRange(DtBegin, DtEnd);
                    if (DtBegin = '-1') or (DtEnd = '-1') then exit;
                    frmWait.lblWait.Caption :=
                    'Processing Stop Codes, this could take a while.  Please wait.'
                 end;

           2  :  begin
                    source := 'APPT^';
                    SetDateRange(DtBegin, DtEnd);
                    if (DtBegin = '-1') or (DtEnd = '-1') then exit;
                    frmWait.lblWait.Caption :=
                    'Processing appointments, this could take a while.  Please wait.'
                 end;

           3  :  begin
                    source := 'TEAM^';
                    frmWait.lblWait.Caption :=
                       'Processing team, this could take a while.  Please wait.'
                 end;

           4  :  begin
                    notyet;
                 end;
        end;
        source := source +
                  Piece(GetFMDelimRec(cboSource.FMLister,cboSource.ItemIndex), U, 1);{Piece(cboSource.FMLister.Results[cboSource.ItemIndex], U, 1); - PDR}

        Mult['"TEAM"'] := TeamID.TeamIen;
        Mult['"DATE"'] := TodayStr;
        Mult['"SOURCE"'] := source;

        if DtBegin <> emptyStr then
           Mult['"BEGIN"']     := DtBegin
        else
           Mult['"BEGIN"']  := TodayStr;

        if DtEnd <> emptyStr then
           Mult['"END"'] := DtEnd
        else
           Mult['"END"'] := TodayStr;

        if chkType.Checked then
           Mult['"TYPE"'] := '1'
        else
           Mult['"TYPE"'] := '99';

        if chkRestrict.Checked then
           Mult['"RESTRICT"'] := '1'
        else
           Mult['"RESTRICT"'] := '0';

     end;

  frmWait.Show;
  application.ProcessMessages;
  RPCBrokerMain.Call;
  frmWait.close;

  x := RPCBrokerMain.Results[1];
  RPCBrokerMain.Results.Delete(1);
  MessageDlg(Piece(x, U, 1) + ' entries were processed!', mtInformation, [mbOk], 0);
  lblProcessed.Caption := Piece(x, U, 1);

  if Piece(RPCBrokerMain.Results[0], U, 1) <> '1' then
     begin
        errlog := TScError.Create;
        errlog.LoadandDisplay(RPCBrokerMain.Results);
        errlog.free;
     end;

  if Piece(x, U, 1) <> emptystr then
     if Piece(x, U, 1) <> '0' then
        frmMain.StatusMsg('Patient assignments filed...', true)
     else
        messagedlg('No patient assignments were filed.', mtInformation, [mbOk], 0);

end;

{=======================================================}
{      Multiple Position Assignment Procedures          }
{=======================================================}

procedure TfrmMltAssign.AssignToPosition;
begin
  DstLabel.Caption := 'New Assignments';
  SrcLabel.Caption := 'Available to Assign';
  chkrestrict.Visible := false;  //patch 177
  cboSelectBy.Items.Addstrings(lstPositionSet2.Items);
  cboSelectBy.ItemIndex := 0;
  cboSelectClick(nil);
  //patch 177 with grpOptions2 do begin
     //left  := grpOptions1.left;
     //top   :=  grpOptions1.top;
     //visible := true;
  //end;
  SelectPosition;
  Self.Caption := 'Multiple Patient Assignments to: ' +  TeamID.PositionName +
                  ' in ' + TeamID.TeamName;
end;

procedure TfrmMltAssign.SelectPosition;
begin
  {if (TeamID.TeamIen = '') or (TeamID.PositionIen = '') then
     close;}
end;

procedure TfrmMltAssign.FilePosAssignments( jobID : string);
var
  ErrLog : TScError;
  x      : string;
begin
  if not ConfirmPosSave(dstList.items.count) then exit;

  frmWait.lblWait.Caption := 'Please wait, processing your selections';
  frmWait.Show;
  application.ProcessMessages;
  RPCBrokerMain.remoteProcedure := 'SC FILE PAT POS ASGN';
  //bp/cmf 177 remove//RPCBrokerMain.RPCVersion := '1'; 
  with RPCBrokerMain.Param[0] do
     begin
        Value := '.SC';
        Ptype := list;

        Mult['"JOB"']       := jobID;
        Mult['"TEAM"']      := TeamID.TeamIen;
        Mult['"POSITION"']  := TeamID.PositionIen;
        Mult['"DATE"']      := TodayStr;
        IF chktype.checked then mult['"TYPE"'] := Inttostr(1) else
        mult['"TYPE"'] := Inttostr(0);
        // patch 177 Mult['"TYPE"']      := IntToStr(grpOptions2.ItemIndex);
     end;

  RPCBrokerMain.Call;
  frmWait.close;
  if RPCBrokerMain.Results[0] = '0' then
   begin
    MessageDlg('Filer NOT queued on server!',mtWarning,[mbOK],0);
 	lblProcessed.Caption := '0';
   end
  else
   frmMain.StatusMsg('Patient assignments filed...',true);

  {x := RPCBrokerMain.Results[1];
  RPCBrokerMain.Results.Delete(1);
  lblProcessed.Caption := Piece(x, U, 1);

  if Piece(RPCBrokerMain.Results[0], U, 1) <> '1' then
     begin
        errlog := TScError.Create;
        errlog.LoadandDisplay(RPCBrokerMain.Results);
        errlog.free;
     end;

  if Piece(x, U, 1) <> emptystr then
     if Piece(x, U, 1) <> '0' then
        frmMain.StatusMsg('Patient assignments filed...', true); }

end;

procedure TfrmMltAssign.FileAllPosAssignments;
var
  ErrLog : TScError;
  x, source      : string;
  DtBegin, DtEnd : TScStr20;
begin
  DtBegin := emptystr;    DtEnd := emptyStr;

  RPCBrokerMain.remoteProcedure := 'SC FILE ALL PAT POS ASGN';
  with RPCBrokerMain.Param[0] do
     begin
        Value := '.SC';
        Ptype := list;

        case cboSelectBy.ItemIndex of
           0  :  begin
                    source := 'PCLN^';
                    frmWait.lblWait.Caption :=
                        'Processing clinics, this could take a while.  Please wait.'
                 end;

           1  :  begin
                    source := 'PSTOPC^';
                    SetDateRange(DtBegin, DtEnd);
                    if (DtBegin = '-1') or (DtEnd = '-1') then exit;
                    frmWait.lblWait.Caption :=
                       'Processing stop codes, this could take a while.  Please wait.'
                 end;

           2  :  begin
                    source := 'PAPPT^';
                    SetDateRange(DtBegin, DtEnd);
                    if (DtBegin = '-1') or (DtEnd = '-1') then exit;
                    frmWait.lblWait.Caption :=
                       'Processing appointments, this could take a while.  Please wait.'
                 end;

           3  :  begin
                    source := 'PTEAM^';
                    frmWait.lblWait.Caption :=
                       'Processing team, this could take a while.  Please wait.'
                 end;

           4  :  begin
                    notyet;
                 end;

           5  :  begin
                    source := 'PPOS^';
                    frmWait.lblWait.Caption :=
                       'Processing position, this could take a while.  Please wait.'
                 end;
        end;
        source := source +
                  Piece(GetFMDelimRec(cboSource.FMLister,cboSource.ItemIndex), U, 1);{Piece(cboSource.FMLister.Results[cboSource.ItemIndex], U, 1); - PDR}

        Mult['"TEAM"'] := TeamID.TeamIen;
        Mult['"POSITION"'] := TeamID.PositionIen;
        Mult['"DATE"'] := TodayStr;
        Mult['"SOURCE"'] := source;
        IF chktype.checked then mult['"TYPE"'] := Inttostr(1)
        else mult['"TYPE"'] := Inttostr(0);
        // patch 177 Mult['"TYPE"']  := IntToStr(grpOptions2.ItemIndex);

        if DtBegin <> emptyStr then
           Mult['"BEGIN"']     := DtBegin
        else
           Mult['"BEGIN"']  := TodayStr;

        if DtEnd <> emptyStr then
           Mult['"END"'] := DtEnd
        else
           Mult['"END"'] := TodayStr;
     end;

  frmWait.Show;
  application.ProcessMessages;
  RPCBrokerMain.Call;
  frmWait.close;

  x := RPCBrokerMain.Results[1];
  RPCBrokerMain.Results.Delete(1);
  MessageDlg(Piece(x, U, 1) + ' entries were processed!', mtInformation, [mbOk], 0);
  lblProcessed.Caption := Piece(x, U, 1);

  if Piece(RPCBrokerMain.Results[0], U, 1) <> '1' then
     begin
        errlog := TScError.Create;
        errlog.LoadandDisplay(RPCBrokerMain.Results);
        errlog.free;
     end;

  if Piece(x, U, 1) <> emptystr then
     if Piece(x, U, 1) <> '0' then
        frmMain.StatusMsg('Patient assignments filed...', true)
     else
        messagedlg('No patient assignments were filed.', mtInformation, [mbOk], 0);

end;

function TfrmMltAssign.ConfirmTeamSave(cnt : integer) : boolean;
var
  cur, max : string[10];
begin
  getTeamInfo.IENS := TeamID.TeamIen;
  getTeamInfo.GetData;

  cur := Piece(GetFMDelimRec(getTeamInfo,3), U, 5);{Piece(getTeamInfo.Results[3], U, 5); - PDR}
  if cur = emptystr then cur := '0';

  max := Piece(GetFMDelimRec(getTeamInfo,1), U, 5);{Piece(getTeamInfo.Results[1], U, 5); - PDR}
  if max = emptystr then max := '0';
  if (pos(',',max)>0) then
     repeat
           max := copy(max,1,pos(',',max)-1)+copy(max,pos(',',max)+1,99) ;
     until pos(',',max)=0 ;

  result := CheckTeamMax(StrToInt(cur) + cnt, StrToInt(max),'-');
end;

function TfrmMltAssign.ConfirmPosSave( cnt : integer) : boolean;
var
  cur, max : string[10];
  i : integer;
begin
  getPosAsgnInfo.IENS := TeamId.PositionIen;
  getPosAsgnInfo.GetData;

  cur := Piece(GetFMDelimRec(getPosAsgnInfo,2), U, 5);{Piece(getPosAsgnInfo.Results[2], U, 5); - PDR}
  if cur = emptystr then cur := '0';

  max := Piece(GetFMDelimRec(getPosAsgnInfo,1), U, 5);{Piece(getPosAsgnInfo.Results[1], U, 5); - PDR}
  if max = emptystr then max := '0';
  if (pos(',',max)>0) then
     repeat
           max := copy(max,1,pos(',',max)-1)+copy(max,pos(',',max)+1,99) ;
     until pos(',',max)=0 ;

  result :=  CheckPositionMax(StrToInt(cur) + cnt, StrToInt(max),'-');
end;

procedure TfrmMltAssign.Print1Click(Sender: TObject);
var
   i: integer;
  tf: TextFile;
begin
  if PrintDialog1.Execute then
    begin
      AssignPrn(tf);
      try
        Screen.Cursor := crHourglass;
        Rewrite(tf);
        {Printer.Canvas.Font.Assign(memErrors.Font);}
        if (Sender as TMenuItem).Name = 'Print1' then begin
           writeln(tf, 'Available to Select');
           writeln(tf, '===================');
           writeln(tf,'');
           for i := 0 to SrcList.items.Count -1 do
              writeln(tf, SrcList.items[i]);
        end;

        if (Sender as TMenuItem).Name = 'Print2' then begin
           writeln(tf, 'Selected to Assign');
           writeln(tf, '===================');
           writeln(tf,'');
           for i := 0 to DstList.items.Count -1 do
              writeln(tf, DstList.items[i]);
        end;

      finally
        CloseFile(tf);
        Screen.Cursor := crDefault;
      end;
    end;
end;

procedure TfrmMltAssign.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (key = VK_F10) and (Shift = [ssShift])  then
        sendMessage(Handle, WM_RBUTTONDOWN, 0, 0)
end;

procedure TfrmMltAssign.SrcListKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (key = VK_F10) and (Shift = [ssShift])  then
        sendMessage(Handle, WM_RBUTTONDOWN, 0, 0)
end;

procedure TfrmMltAssign.DstListKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (key = VK_F10) and (Shift = [ssShift])  then
        sendMessage(Handle, WM_RBUTTONDOWN, 0, 0)
end;

procedure TfrmMltAssign.FormDestroy(Sender: TObject);
begin
  {TeamID.Free;}
end;

procedure TfrmMltAssign.HelpBtnClick(Sender: TObject);
begin
  if FormMode =AssignTM then
   Application.HelpContext(70100)
  else
   Application.HelpContext(70110);
end;

function TfrmMltAssign.TodayStr: string;
begin
 result := StrToFMDate(FormatDateTime('mm/dd/yyyy', Now));
end;


end.
