{*******************************************************************}
{                                                                   }
{ Scheduling Report utility unit for Primary Care Management        }
{ a component of the Scheduling Redesign Project                    }
{                                                                   } 
{ Scheduling ver 5.3 AUG 13, 1993                                   }      
{ Management Information Products Group                             }
{ Albany NY (mjk)                                                   }
{ Dept. of Veteran Affairs                                          }
{                                                                   }
{ Version: 1T15     Updated:                                        }
{                                                                   }
{*******************************************************************}

unit Rptutl;

interface

uses Sysutils, graphics, trpcb, mfunstr, classes, Dialogs, scutdef, scut3;

type
  TScFileType        = (division, team, practitioner, role, clinic, userclass);
  TScSelAllowed      = (oman, one, oma);
  TScAccessLevel     = (alPrivate, alPublic);
  TScValidateMode    = (vmFull, vmSelections);

  {Basic 'file entry needed' class definition}
  TScRptFile = class(TScObject)
  private
    FFileType    : TScStr20;
    FFileNumber  : TScStr20;
    FSelAllowed  : TScSelAllowed;
  public
    property FileType    : TScStr20        read FFileType   write FFileType;
    property FileNumber  : TScStr20        read FFileNumber write FFileNumber;
    property SelAllowed  : TScSelAllowed  read FSelAllowed write FSelAllowed;
  end;

  {Basic 'field to prompt for' class definition}
  TScRptField = class(TObject)
  private
    FComponentName    : TScStr50;
    FAlwaysAsk        : Boolean;
    FRequired         : Boolean;
    FDefaultValue     : TScStr100;
  public
    property ComponentName : TScStr50  read FComponentName write FComponentName;
    property AlwaysAsk     : Boolean   read FAlwaysAsk     write FAlwaysAsk;
    property Required      : Boolean   read FRequired      write FRequired;
    property DefaultValue  : TScStr100 read FDefaultValue  write FDefaultValue;
  end;

  {Basic 'report defintion' class definition}
  TScRptDef = class(TScObject)
  private
    FName       : TScStr50;
    FID         : TScStr20;
    FRptGroup   : TScStr50;
    FDescription: TStrings;
    FSorts      : TStrings;
    FFields     : TStrings;
    FFiles      : TStrings;
    procedure LoadDescription(RPCBroker: TRPCBroker ; var item: Integer);
    procedure LoadSorts(RPCBroker: TRPCBroker ; var item: Integer);
    procedure LoadFields(RPCBroker: TRPCBroker ; var item: Integer);
    procedure LoadFiles(RPCBroker: TRPCBroker ; var item: Integer);

  public
    constructor Create;
    destructor  Destroy; override;
    procedure   ClearRecord;
    function    GetRecord(RPCBroker: TRPCBroker; RecID: TScStr20): Boolean;
    property Name        : TScStr50 read FName        write FName;
    property ID          : TScStr20 read FID          write FID;
    property RptGroup    : TScStr50 read FRptGroup    write FRptGroup;
    property Description : TStrings read FDescription write FDescription;
    property Sorts       : TStrings read FSorts       write FSorts;
    property Fields      : TStrings read FFields      write FFields;
    property Files       : TStrings read FFiles       write FFiles;
  end;

  {Basic 'file entry selection' class definition}
  TScQrySelection = class(TObject)
  private
    FDisplayText : TScStr50;
    FFileType    : TScStr20;
    FID          : TScStr20;
  public
    function GetSelection(RPCBroker: TRPCBroker; SelectionData: TStringList): Boolean;
    procedure CopyFrom(FromSelection: TScQrySelection);
    property DisplayText : TScStr50 read FDisplayText write FDisplayText;
    property FileType    : TScStr20 read FFileType    write FFileType;
    property ID          : TScStr20 read FID          write FID;
  end;

  {Basic query fields value}
  TScQryField = class(TObject)
  private
    FComponentName : TScStr50;
    FValue         : TScStr100;
  public
    property ComponentName : TScStr50   read FComponentName write FComponentName;
    property Value         : TScStr100  read FValue         write FValue;
  end;

  {Basic report query definition}
  TScQryDef = class(TSCObject)
  private
    FName         : TScStr50;
    FID           : TScStr20;
    FCreator      : TScStr50;
    FCreatorID    : TScStr20;
    FAccessLevel  : TScAccessLevel;
    FReport       : TScStr50;
    FReportID     : TScStr20;
    FLastSaveDate  : TScStr50;
    FLastSaveDT    : TScStr20;
    FModified     : Boolean;
    FDescription  : TStrings;
    FFields       : TStrings;
    FSelections   : TList;
    procedure LoadDescription(RPCBroker: TRPCBroker ; var item: Integer);
    procedure LoadFields(RPCBroker: TRPCBroker ; var item: Integer);
    procedure LoadSelections(RPCBroker: TRPCBroker ; var item: Integer);

  public
    constructor Create;
    destructor  Destroy; override;
    procedure   BuildQuery(var RPCBroker: TRPCBroker; node: integer);
    procedure   NewQuery(rpt: TScRptDef; user: TScUser);
    function    GetQuery(RPCBroker: TRPCBroker; QueryID: TScstr20): Boolean;
    function    ValidateQuery(var RPCBroker: TRPCBroker; ValMode: TScValidateMode): Boolean;
    function    SaveQuery(RPCBroker: TRPCBroker): Boolean;
    function    DeleteQuery(RPCBroker: TRPCBroker): Boolean;
    function    ReportPrint(RPCBroker: TRPCBroker ;
                                      Printer  : String ;
                                      RunDate  : String ;
                                      RunTime  : String): String;
    procedure   ClearQuery;
    procedure   ClearFields;
    procedure   ClearSelections;
    property Name         : TScStr50       read FName         write FName;
    property ID           : TScStr20       read FID           write FID;
    property Creator      : TScStr50       read FCreator      write FCreator;
    property CreatorID    : TScStr20       read FCreatorID    write FCreatorID;
    property AccessLevel  : TScAccessLevel read FAccessLevel  write FAccessLevel;
    property Report       : TScStr50       read FReport       write FReport;
    property ReportID     : TScStr20       read FReportID     write FReportID;
    property LastSaveDate : TScStr50       read FLastSaveDate write FLastSaveDate;
    property LastSaveDT   : TScStr20       read FLastSaveDT   write FLastSaveDT;
    property Modified     : Boolean        read FModified     write FModified;
    property Description  : TStrings       read FDescription  write FDescription;
    property Fields       : TStrings       read FFields       write FFields;
    property Selections   : TList          read FSelections   write FSelections;
  end;

{constants used throughout autolink functionality.
 Some may be useful in other areas.}
const
  u                  = '^';
  TimerOn            = True;
  TimerOff           = False;
  END_OF_SECTION     = '$$END';
  FileType: array[TScFileType] of TScStr20 = ('DIVISION', 'TEAM','PRACTITIONER','ROLE',
                                               'CLINIC', 'USERCLASS');


implementation

uses pcmm1;

{       ----------  TScRptDef Methods  ----------}

{Creates the report definition object instance.}
constructor TScRptDef.Create;
begin
  inherited Create;
  FDescription := TStringList.Create;
  FSorts       := TStringList.Create;
  FFields      := TStringList.Create;
  FFiles       := TStringList.Create;
end;

{Frees the report definition object instance.}
destructor TScRptDef.Destroy;
begin
  ClearRecord;
  FDescription.Free;
  FSorts.Free;
  FFields.Free;
  FFiles.Free;
  inherited destroy;
end;

{Frees objects attached to FFields, FFiles TLists}
procedure TScRptDef.ClearRecord;
var
   i: integer;
begin
  for i :=0 to FFields.Count - 1 do
    TScRptField(FFields.Objects[i]).Free;

  for i :=0 to FFiles.Count - 1 do
    TScRptFile(FFiles.Objects[i]).Free;

  FDescription.Clear;
  FSorts.Clear;
  FFields.Clear;
  FFiles.Clear;
end;

{Retrieves data from server for team record assoicated with RecID.}
function TScRptDef.GetRecord(RPCBroker: TRPCBroker; RecID: TScStr20): Boolean;
var
  x: string;
  i,j, cnt: integer;
begin
  FID := RecID;
  ClearRecord;
  with RPCBroker do
    begin
      RemoteProcedure := 'SCRP DEFINITION GETRECORD';
      Param[0].PType := literal;
      Param[0].Value := FID;
      Call;
    end;
  cnt := RPCBroker.Results.Count;
  if cnt > 0 then
    begin
      GetRecord := True;
      x := RPCBroker.Results[0];
      FName := piece(x,u,1);
      for i := 1 to cnt - 1 do
        begin
          j := i ;
          x := RPCBroker.Results[j];
          if      x = '[Description]' then
            LoadDescription(RPCBroker, j)
          else if x = '[Sorts]' then
            LoadSorts(RPCBroker, j)
          else if x = '[Fields]' then
            LoadFields(RPCBroker, j)
          else if x = '[Files]' then
            LoadFiles(RPCBroker, j)
        end;
    end
  else
    GetRecord := False;
end;

procedure TScRptDef.LoadDescription(RPCBroker: TRPCBroker ; var item: Integer);
var
   x       : String;
begin
  x := '';
  while (x <> END_OF_SECTION) and (item <> RPCBroker.Results.Count - 1) do
    begin
      inc(item);
      x := RPCBroker.Results[item];
      if x <> END_OF_SECTION then
        FDescription.Add(x);
    end;
end;

procedure TScRptDef.LoadSorts(RPCBroker: TRPCBroker ; var item: Integer);
var
   x       : String;
begin
  x := '';
  while (x <> END_OF_SECTION) and (item <> RPCBroker.Results.Count - 1) do
    begin
      inc(item);
      x := RPCBroker.Results[item];
      if x <> END_OF_SECTION then
        FSorts.Add(x);
    end;
end;

procedure TScRptDef.LoadFields(RPCBroker: TRPCBroker ; var item: Integer);
var
   x   : String;
   fld : TScRptField;
begin
  x := '';
  while (x <> END_OF_SECTION) and (item <> RPCBroker.Results.Count - 1) do
    begin
      inc(item);
      x := RPCBroker.Results[item];
      if x <> END_OF_SECTION then
        begin
          fld               := TScRptField.Create;
          fld.ComponentName :=  piece(x, u, 1);
          fld.Required      := (piece(x, u, 2) = 'YES');
          fld.AlwaysAsk     := (piece(x, u, 3) = 'YES');
          fld.Defaultvalue  :=  piece(x, u, 4);
          FFields.AddObject(fld.ComponentName, fld);
        end;
    end;
end;

procedure TScRptDef.LoadFiles(RPCBroker: TRPCBroker ; var item: Integer);
var
   x   : String;
   fle : TScRptFile;
begin
  x := '';
  while (x <> END_OF_SECTION) and (item <> RPCBroker.Results.Count - 1) do
    begin
      inc(item);
      x := RPCBroker.Results[item];
      if x <> END_OF_SECTION then
        begin
          fle            := TScRptFile.Create;
          fle.FileType   :=  piece(x, u, 1);
          fle.FileNumber :=  piece(x, u, 2);
          fle.SelAllowed :=  TScSelAllowed(StrToInt(piece(x, u, 3)));
          FFiles.AddObject(fle.FileType, fle);
        end;
    end;
end;

{       ----------  TScQrySelection Methods  ----------}

{Retrieves basic data from server about link that is not available
 in autolink object. Currently, it retrieves list of teams that the
 link entity is linked to.}
function TScQrySelection.GetSelection(RPCBroker: TRPCBroker; SelectionData: TStringList): Boolean;
var
  x: string;
  i: integer;
begin
  with RPCBroker do
    begin
      RemoteProcedure := 'SCRP FILE ENTRY GETSELECTION';
      Param[0].PType := literal;
      Param[0].Value := FFileType;
      Param[1].PType := literal;
      Param[1].Value := FID;
      Call;
      if Results.Count > 0 then
         begin
           GetSelection := True;
           SelectionData.Assign(Results);
         end
      else
         GetSelection := False;
    end;

end;

procedure TScQrySelection.CopyFrom(fromSelection: TScQrySelection);
begin
  FDisplayText := fromSelection.DisplayText;
  FFileType    := fromSelection.FileType;
  FID          := fromSelection.ID;
end;

{       ----------  TScQryDef Methods  ----------}

constructor TScQryDef.Create;
begin
  inherited Create;
  FDescription := TStringlist.Create;
  FFields      := TStringList.Create;
  FSelections  := TList.Create;
end;

destructor TScQryDef.Destroy;
begin
  ClearQuery;
  FDescription.Free;
  FFields.Free;
  FSelections.Free;
  inherited Destroy;
end;

procedure TScQryDef.ClearQuery;
begin
  FDescription.Clear;
  ClearSelections;
  ClearFields;
end;

procedure TScQryDef.ClearSelections;
var
   i: integer;
begin
  for i :=0 to (FSelections.Count - 1) do
    TScQrySelection(FSelections[i]).Free;
  FSelections.Clear;
end;

procedure TScQryDef.ClearFields;
var
  i: integer;
begin
  for i :=0 to (FFields.Count - 1) do
    TScQryField(FFields.Objects[i]).Free;
  FFields.Clear;
end;

procedure TScQryDef.NewQuery(rpt: TScRptDef; user: TScUser);
var
  i    : integer;
  Rfld : TScRptField;
  Qfld : TScQryField;
begin
  ClearQuery;
  FID           := '+1';
  FName         := 'New Query for ' + user.Name;
  FCreator      := user.Name;
  FCreatorID    := user.ID;
  FAccessLevel  := alPrivate;
  FReport       := '';
  FReportID     := '';
  FLastSaveDate := '';
  FLastSaveDT   := '';
  FModified     := False;

  if rpt <> nil then
    begin
      FReport   := rpt.Name;
      FReportID := rpt.ID;
      if rpt.Fields.Count > 0 then
        for i := 0 to Rpt.Fields.Count - 1 do
          begin
            Rfld := TScRptField(rpt.Fields.Objects[i]);
            Qfld := TScQryField.Create;
            Qfld.ComponentName := rpt.Fields[i];
            if Rfld.DefaultValue <> '' then
              Qfld.Value := Rfld.DefaultValue;
            FFields.AddObject(rpt.Fields[i],Qfld);
          end;
    end;
end;

{Retrieves data from server for query record assoicated with RecID.}
function TScQryDef.GetQuery(RPCBroker: TRPCBroker; QueryID: TScStr20): Boolean;
var
  x,y : string;
  i,j, cnt: integer;
begin
  FID := QueryID;
  ClearQuery;
  with RPCBroker do
    begin
      RemoteProcedure := 'SCRP QUERY GETRECORD';
      Param[0].PType := literal;
      Param[0].Value := QueryID;
      Call;
    end;
  cnt := RPCBroker.Results.Count;
  if cnt > 1 then
    begin
      GetQuery := True;
      x := RPCBroker.Results[0];
      y := RPCBroker.Results[1];
      FName         := piece(x, u, 1);
      FCreator      := piece(y, u, 2);
      FCreatorID    := piece(x, u, 2);
      FAccessLevel  := TScAccessLevel(piece(y, u, 3)='PUBLIC');
      FReport       := piece(y, u, 4);
      FReportID     := piece(x, u, 4);
      FLastSaveDT   := piece(x, u, 5);
      FLastSaveDate := FMDateToStr(FLastSaveDT) ;
      FModified     := False;
      for i := 2 to cnt - 1 do
        begin
          j := i ;
          x := RPCBroker.Results[j];
          if      x = '[Description]' then
            LoadDescription(RPCBroker, j)
          else if x = '[Fields]' then
            LoadFields(RPCBroker, j)
          else if x = '[Selections]' then
            LoadSelections(RPCBroker, j)
        end;
    end
  else
    GetQuery := False;
end;

procedure TScQryDef.LoadDescription(RPCBroker: TRPCBroker ; var item: Integer);
var
   x       : String;
begin
  x := '';
  while (x <> END_OF_SECTION) and (item <> RPCBroker.Results.Count - 1) do
    begin
      inc(item);
      x := RPCBroker.Results[item];
      if x <> END_OF_SECTION then
        FDescription.Add(x);
    end;
end;

procedure TScQryDef.LoadFields(RPCBroker: TRPCBroker ; var item: Integer);
var
   x   : String;
   fld : TScQryField;
begin
  x := '';
  while (x <> END_OF_SECTION) and (item <> RPCBroker.Results.Count - 1) do
    begin
      inc(item);
      x := RPCBroker.Results[item];
      if x <> END_OF_SECTION then
        begin
          fld               := TScQryField.Create;
          fld.ComponentName :=  piece(x, u, 1);
          fld.Value         :=  piece(x, u, 2);
          FFields.AddObject(fld.ComponentName, fld);
        end;
    end;
end;

procedure TScQryDef.LoadSelections(RPCBroker: TRPCBroker ; var item: Integer);
var
   x   : String;
   sel : TScQrySelection;
begin
  x := '';
  while (x <> END_OF_SECTION) and (item <> RPCBroker.Results.Count - 1) do
    begin
      inc(item);
      x := RPCBroker.Results[item];
      if x <> END_OF_SECTION then
        begin
          sel              := TScQrySelection.Create;
          sel.FDisplayText :=  piece(x, u, 1);
          sel.FFileType    :=  piece(x, u, 2);
          sel.FID          :=  piece(x, u, 3);
          FSelections.Add(sel);
        end;
    end;
end;

function TScQryDef.ValidateQuery(var RPCBroker: TRPCBroker; ValMode: TScValidateMode): Boolean;
var
   scok : Boolean;
   vm   : TScStr20;
begin
  RPCBroker.RemoteProcedure := 'SCRP QUERY VALIDATE';
  RPCBroker.Param[0].PType := literal;
  if ValMode = vmSelections then
    vm := 'SELECTIONS'
  else
    vm := 'FULL';
  RPCBroker.Param[0].Value := vm;
  BuildQuery(RPCBroker, 1);
  RPCBroker.Call;
  scok := (piece(RPCBroker.Results[0], u, 1) = '1');
  if not scok then
      Error.LoadAndDisplay(RPCBroker.Results);
  ValidateQuery := scok;
end;

function TScQryDef.SaveQuery(RPCBroker: TRPCBroker): Boolean;
var
  x    : String;
  scok : Boolean;
begin
  RPCBroker.RemoteProcedure := 'SCRP QUERY SAVE';
  BuildQuery(RPCBroker, 0);
  RPCBroker.Call;
  x := RPCBroker.Results[0];
  scok := (piece(x, u, 1) = '1');
  if scok then
    begin
      if FID = '+1'           then FID := piece(x, u, 2);
      GetQuery(RPCBroker, FID);
    end
  else
      Error.LoadAndDisplay(RPCBroker.Results);
  SaveQuery := scok;
end;

function TScQryDef.DeleteQuery(RPCBroker: TRPCBroker): Boolean;
begin
  RPCBroker.RemoteProcedure := 'SCRP QUERY DELETE';
  BuildQuery(RPCBroker, 0);
  RPCBroker.Call;
  if (piece(RPCBroker.Results[0], u, 1) = '1') then
    DeleteQuery := True
  else
    begin
      Error.LoadAndDisplay(RPCBroker.Results);
      DeleteQuery := False;
    end;
end;

procedure TScQryDef.BuildQuery(var RPCBroker: TRPCBroker; node: integer);
var
    x : String;
cnt,i : integer;
  fld : TScQryField;
  sel : TScQrySelection;

begin
  cnt := 0;
  RPCBroker.Param[node].PType := list;
  {build top level fields}
  if FAccessLevel = alPrivate then
    x := '1'
  else
    x := '2';
  x := FName + u + FCreatorID + u + x + u + FReportID;
  inc(cnt);
  RPCBroker.Param[node].Mult[IntToStr(cnt)] := x;

  if FAccessLevel = alPrivate then
    x := 'PRIVATE'
  else
    x := 'PUBLIC';
  x := FID + u + FCreator + u + x + u + FReport;
  inc(cnt);
  RPCBroker.Param[node].Mult[IntToStr(cnt)] := x;

  {Description Build}
  inc(cnt);
  RPCBroker.Param[node].Mult[IntToStr(cnt)] := '[Description]';
  for i := 0 to FDescription.Count - 1 do
    begin
      inc(cnt);
      RPCBroker.Param[node].Mult[IntToStr(cnt)] := FDescription[i];
    end;
  inc(cnt);
  RPCBroker.Param[node].Mult[IntToStr(cnt)] := END_OF_SECTION;

  {Fields Build}
  inc(cnt);
  RPCBroker.Param[node].Mult[IntToStr(cnt)] := '[Fields]';
  for i := 0 to FFields.Count - 1 do
    begin
      fld := TScQryField(FFields.Objects[i]);
      inc(cnt);
      RPCBroker.Param[node].Mult[IntToStr(cnt)] := fld.ComponentName + u +
                                                   fld.Value;
    end;
  inc(cnt);
  RPCBroker.Param[node].Mult[IntToStr(cnt)] := END_OF_SECTION;

  {Selections Build}
  inc(cnt);
  RPCBroker.Param[node].Mult[IntToStr(cnt)] := '[Selections]';
  for i := 0 to FSelections.Count - 1 do
    begin
      sel := TScQrySelection(FSelections[i]);
      inc(cnt);
      RPCBroker.Param[node].Mult[IntToStr(cnt)] := sel.DisplayText + u +
                                                   sel.FileType    + u +
                                                   sel.ID;
    end;
  inc(cnt);
  RPCBroker.Param[node].Mult[IntToStr(cnt)] := END_OF_SECTION;

end;

function TScQryDef.ReportPrint(RPCBroker: TRPCBroker ;
                               Printer  : String ;
                               RunDate  : String ;
                               RunTime  : String): String;
var
  TaskNum : String;

begin
  with RPCBroker do
    begin
     RemoteProcedure := 'SCRP REPORT PRINT';
     Param[0].PType := literal;
     Param[0].Value := Printer;
     Param[1].PType := literal;
     Param[1].Value := RunDate;
     Param[2].PType := literal;
     Param[2].Value := RunTime;
     BuildQuery(RPCBroker, 3);
     Call;
     TaskNum := piece(Results[0], u, 1);
     if TaskNum = '0' then
       Error.LoadAndDisplay(RPCBroker.Results);
    end;
 ReportPrint := TaskNum;
end;

end.
