{*******************************************************************}
{                                                                   }
{   Staff Member Lookup Primary Care Management                     }
{   Performs a name lookup in VA #200 for staff assignment.         }
{   displays the name, title, and mail code.                        }
{   If the User class file is installed, lookup is restricted       }
{   to the associated user class.                                   }                                                     
{                                                                   }
{    Scheduling ver 5.3 AUG 13, 1993                                }      
{    Management Information Products Group                          }
{    Albany NY (SCK)                                                }
{    Dept. of Veteran Affairs                                       }
{    Feb 1996                                                       }
{                                                                   }
{    Last updated: 28 May 2002 (J. Peterson)                        }
{                                                                   }
{*******************************************************************}

unit Stafflkp;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, Fmcmpnts, Trpcb, OvcBase, OvcVlb, messages, Dialogs,
  SysUtils, Scut3, Scut2, Scutdef, Mfunstr, Scutbrk, Menus;

type
  TfrmStaffLookUp = class(TForm)
    cmdOk: TBitBtn;
    cmdCancel: TBitBtn;
    HelpBtn: TBitBtn;
    LookupList: TOvcVirtualListBox;
    OvcController1: TOvcController;
    txtLookup: TEdit;
    Label1: TLabel;
    PopupMenu1: TPopupMenu;
    Details1: TMenuItem;
    btnSearch: TBitBtn;
    procedure cmdOkClick(Sender: TObject);
    procedure LookupListGetItem(Sender: TObject; Index: Longint;
      var ItemString: String);
    procedure txtLookupExit(Sender: TObject);
    procedure txtLookupKeyPress(Sender: TObject; var Key: Char);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure txtLookupEnter(Sender: TObject);
    procedure LookupListDblClick(Sender: TObject);
    procedure LookupListKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure cmdCancelClick(Sender: TObject);
    procedure txtLookupKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormShow(Sender: TObject);
    procedure RPCBroker1NetError(Sender: TObject; var Error: EBrokerError);
    procedure btnSearchClick(Sender: TObject);
  private
    { Private declarations }
    DataList : TStringList;
    NetError : Boolean;
  public
    { Public declarations }
    LookUpIen: string[25];
    LookUpName: string[30];
    UserClass : string;
    UsrC : string[12];
    pc  :string;
    role : string;
    procedure Pad(var s: string);
    procedure BuildTeamList(list: TStrings);
  end;

var
  frmStaffLookUp: TfrmStaffLookUp;

implementation

uses scptpf, Pcmm1;

{$R *.DFM}

procedure TfrmStaffLookUp.FormCreate(Sender: TObject);
var
  TabArray: array[0..2] of Integer;
begin
  TabArray[0] := 120;
  TabArray[1] := 200;
  LookupList.SetTabStops(TabArray);
  DataList := TStringList.Create;
  DataList.Sorted := true;
  DataList.Duplicates := DupIgnore;
  LookupList.NumItems := 0;
  txtLookup.Text := '';
  NetError := false;
end;

{

}
procedure TfrmStaffLookUp.FormDestroy(Sender: TObject);
begin
  {ClearData}
  DataList.Free;
end;

{

}
procedure TfrmStaffLookUp.cmdOkClick(Sender: TObject);
var
  i: integer;
begin
  i := LookupList.ItemIndex;
  if i < 0 then
     modalResult := mrCancel
  else begin
     LookUpIen := (Piece(datalist[i], TAB, 6));
     LookUpName := (Piece(datalist[i], TAB, 1));
     ModalResult := mrOk;
  end;
end;

{

}
procedure TfrmStaffLookUp.LookupListGetItem(Sender: TObject; Index: Longint;
  var ItemString: String);
begin
  If (index >= 0) and (Index < DataList.Count) then
     ItemString := DataList.Strings[Index];
end;

{


}
procedure TfrmStaffLookUp.txtLookupExit(Sender: TObject);
begin

end;

{

}
procedure TfrmStaffLookUp.BuildTeamList(list: TStrings);
var
  i: integer;
  pn: string;
begin
  for i := 0 to list.count - 1 do begin
     pn := Piece(list[i], U, 6);
     Pad(pn);
     DataList.Add(Copy(pn, 1, 23) + TAB + Piece(list[i], U, 8) + TAB +
                  Piece(list[i], U, 7) + TAB + TAB + TAB +
                  Piece(list[i], U, 1));
     end;

  LookupList.numItems := DataList.Count;
  LookupList.Repaint;
  LookupList.Setfocus;
end;

{

}
procedure TfrmStaffLookUp.Pad(var s: string);
var
  i: integer;
begin
  for i := Length(s) to 25 do
     s := s + ' ';
end;


procedure TfrmStaffLookUp.txtLookupKeyPress(Sender: TObject;
  var Key: Char);
begin
  key := UpCase(key);
end;


procedure TfrmStaffLookUp.txtLookupEnter(Sender: TObject);
begin
  cmdOk.ENabled := false;
  if DataList.Count > 0 then DataList.Clear;
  LookupList.NumItems := 0;
  LookupList.Refresh;
  {txtPatLookup.Text := '';}
end;

procedure TfrmStaffLookUp.LookupListDblClick(Sender: TObject);
begin
  cmdOkClick(nil);
end;

procedure TfrmStaffLookUp.LookupListKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = VK_SPACE) then cmdOkClick(nil);
end;

procedure TfrmStaffLookUp.cmdCancelClick(Sender: TObject);
begin
  ModalResult := mrCancel;
  {close;}
end;

procedure TfrmStaffLookUp.txtLookupKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if key = VK_RETURN then btnSearchClick(Sender);
end;

procedure TfrmStaffLookUp.FormShow(Sender: TObject);
begin
  UsrC := Piece(UserClass, U, 1);
  if UsrC = '-1' then
     Caption := 'Staff lookup'
  else
     Caption := 'Staff Lookup - User Class: ' + Piece(UserClass, U, 2);

  DataList.Clear;
  LookupList.NumItems := 0;
  txtLookup.Text := '';
  txtLookUp.SetFocus;
end;

procedure TfrmStaffLookUp.RPCBroker1NetError(Sender: TObject;
  var Error: EBrokerError);
var
  msg: string;
begin
  msg := 'A Network error has been encountered.  Please make' + RETURN +
         'a note of your current entry.  You will need to' + RETURN +
         'logon to DHCP before continuing';
  messageDlg(msg, mtError, [mbOk], 0);
  NetError := true;
  Halt;
end;

procedure TfrmStaffLookUp.btnSearchClick(Sender: TObject);
var
  i, ssn, spcl: integer;
  s, s1, Index: string[10];
  lst: TStringList;
begin
  CmdOk.Enabled := true;
  lst := TStringList.Create;

  try
     if (ActiveControl = cmdCancel) then exit;

     Screen.Cursor := crHourglass;

     if txtLookUp.text = '' then begin
        Screen.Cursor := crDefault;
        exit;
        end;

     if (Length(txtLookup.Text) < 3) and (UsrC = '-1') then begin
        Screen.Cursor := crDefault;
        Application.NormalizeTopMosts   ;
        messagedlg('Must enter at least 3 characters.', mtInformation, [mbOk], 0);
        Application.RestoreTopMosts   ;
        txtLookUp.setfocus;
        exit;
        end
     else
        s := txtLookup.Text;

     if not (UsrC = '-1') then
        GetStaffList(RPCBrokerMain, '1', UsrC, s, '',ROLE, lst)
     else
        GetStaffList(RPCBrokerMain, '0', '', s, '',ROLE, lst);

     if lst.count > 0 then
        BuildTeamList(lst)
     else begin
        Application.NormalizeTopMosts   ;
        messagedlg('No matching entries were found', mtInformation, [mbOk], 0);
        Application.RestoreTopMosts   ;
        txtLookup.setfocus;
     end;

  finally
     lst.free;
     Screen.Cursor := crDefault;
  end;
end;

end.
