{*****************************************************************}
{                 General Definitions for                         }
{                      Scheduling                                 }
{                                                                 }
{                     7 November 95                               }
{                         ver 1T9b                                }
{*****************************************************************}

unit Scutdef;

interface

uses  SysUtils, WinTypes, WinProcs, Messages, Classes, Forms, Trpcb,
      MfunStr;


Const
  U = '^';
  CRLF = #13#10;

type
  ENoTeam = class(Exception);
  EInvalidName = class(Exception);
  EPositions = class(Exception);
  EHistory = class(Exception);
  EInvalidPosition = class(Exception);
  EStaffError = class(Exception);
  EIniError = class(Exception);
  EDeletionError = class(Exception);
  EGeneralError = class(Exception);
  EInsufficientMemory = class(Exception);

  TMultFormMode = (AssignTm, DischargeTm, AssignPos, DischargePos);

  TStatusType = (INACTIVE, ACTIVE);

  TRecType = (TTeam, TPos, TStf, TPatTeam, TPatPos, TTmHistory, TPsHistory);

  TEditMode = (AddItem, EditItem, ViewItem);

  TPcmmMsg = (EditStaff, AddStaff, DelStaff, InactStaff);

  TScStr20           = String[20];
  TScStr50           = String[50];
  TScStr100          = String[100];

  TStaff = Record
     Name: string[30];
     Index: integer;
     HistoryIen: string[15];
     EffDate: String[12]; {Fileman date format}
     Reason: string[50];
     Status: string[2];
  end;

  TTeamType = (Outpatient, Inpatient);

  TProfile = record
     PositionIen: string[10];
     TeamIen: string[10];
     PositionName: string[30];
     TeamEffDate: TDateTime;
     PositionEffDate: TDateTime;
     Preceptor      : String[50];
     NewRecord: Boolean;
     Parent: TForm;
     Handle: THandle;           { Remove}
     TeamName: string;
     TmClassification: TTeamType;
     Duz: string[10];
     UserClass: string;
     PC:string[1];
     //bp/cmf 177
     Link:             Boolean;
     LinkActive:       Boolean;
     FutureAsgn:       Boolean;
     DeleteKey:        Boolean;
     PreceptorB:       Boolean;
     TeamCanProvidePC: Boolean;
     StaffShortCut:    Boolean;
     //bp/cmf 177
  end;

  TTeamAsgn = record
     TeamName: string[30];
     dfn: string[15];
     Patient: string[30];
     TeamIen: string[15];
     EffDate: string[25];
     TeamAsgn: string[25];
     Parent: Tform;
     Mode: TEditMode;
     PrimaryCare: Boolean;
  end;


 TBrokerInfo = record
     PortID      : string;
     ServerName  : string;
     ClientTimeOut: integer;//bp/cmf 204

  end;


  TAssignmentData = class(TObject)
  private
    FTeamName       : TScStr50;
    FTeamIen        : TScStr20;
    FPositionName   : TScStr50;
    FPositionIen    : TScStr20;
    FMode           : TEditMode;
  public
    property TeamName     : TScStr50 read FTeamName  write FTeamName;
    property TeamIen      : TScStr20 read FTeamIen   write FTeamIen;
    property PositionName : TScStr50 read FPositionName write FPositionName;
    property PositionIen  : TScStr20 read FPositionIen write FPositionIen;
    property Mode         : TEditMode read FMode write FMode;
  end;


  {Basic 'error' class definition}
  TScError = class(TObject)
  private
    { Private declarations }
    FECode     : TScStr20;
    FProcess   : TScStr50;
    FErrorCnt  : TScStr20;
    FLineCnt   : TScStr20;
    FErrorList : TStrings;
  public
    { Public declarations }
    constructor Create;
    destructor  Destroy; override;
    procedure   LoadErrors(list: TStrings);
    procedure   DisplayErrors;
    procedure   LoadAndDisplay(list: TStrings);
    property ECode     : TScStr20 read FECode     write FECode;
    property Process   : TScStr50 read FProcess   write FProcess;
    property ErrorCnt  : TScStr20 read FErrorCnt  write FErrorCnt;
    property LineCnt   : TScStr20 read FLineCnt   write FLineCnt;
    property ErrorList : TStrings read FErrorList write FErrorList;
  end;

  {Basic TSc* object}
  TScObject = class(TObject)
  private
    FError          : TScError;
  public
    constructor Create;
    destructor  Destroy; override;
    property Error : TScError read FError write FError;
  end;

  {Basic user record}
  TScUser = class(TScObject)
  private
    FName           : TScStr50;
    FID             : TScStr20;
    FDefaultQryID   : TScStr20;
    FDefaultQryName : TScStr50;
    FUci            : TScStr20;      { This envrionment data was added as a   }
    FVolume         : TScStr20;      { temporary means of displaying this data}
    FNode           : TScStr20;      { to the user until an environment object}
    FBoxLookup      : TScStr20;      { could be created and placed into use   }
    FInstitution    : TScStr50;      {Added 1/31/97 - RV}
  public
    function LoadCurrentUser(RPCBroker: TRPCBroker): Boolean;
    function LoadUser(RPCBroker: TRPCBroker; UserID: TScStr20): Boolean;
    function CheckQueryName(RPCBroker: TRPCBroker; QryName: TScStr50): TScStr20;
    property Name           : TScStr50 read FName           write FName;
    property ID             : TScStr20 read FID             write FID;
    property DefaultQryID   : TScStr20 read FDefaultQryID   write FDefaultQryID;
    property DefaultQryName : TScStr50 read FDefaultQryName write FDefaultQryName;
    property Uci            : TScStr20 read FUci            write FUci;
    property Volume         : TScStr20 read FVolume         write FVolume;
    property Node           : TScStr20 read FNode           write FNode;
    property BoxLookup      : TScStr20 read FBoxLookup      write FBoxLookup;
    property DefaultInst    : TScStr50 read FInstitution    write FInstitution;  {Added 1/31/97 - RV}
  end;

Const
     RETURN = #13;
     NEW_ENTRY = '+1';
     TAB = #9;
     UNDEFINED_NODE = '{{0}}';
     DEFINED_NODE = '{{10}}';
     NONE = '< NONE >';
     MEM_AVAIL = 4000;
     NULL_DATE = '  /  /    ';
     Month = 'JAN^FEB^MAR^APR^MAY^JUN^JUL^AUG^SEP^OCT^NOV^DEC';


implementation
uses scuterr;
{       ----------  TScObject Methods  ----------}

{Creates the report definition object instance.}
constructor TScObject.Create;
begin
  inherited Create;
  FError       := TScError.Create;
end;

{Frees the report definition object instance.}
destructor TScObject.Destroy;
begin
  FError.Free;
  inherited destroy;
end;


{       ----------  TScUser Methods  ----------}
function TScUser.LoadCurrentUser(RPCBroker: TRPCBroker): Boolean;
begin
 LoadCurrentUser := LoadUser(RPCBroker, 'CURRENT USER');
end;

function TScUser.LoadUser(RPCBroker: TRPCBroker; UserID: TScStr20): Boolean;
var
 scok : Boolean;
    x : string;
begin
  FID := '';
  FName := '';
  with RPCBroker do
    begin
      RemoteProcedure := 'SCUT GET USER RECORD';
      Param[0].PType  := literal;
      Param[0].Value  := UserID;
      Call;
      x               := Results[0];
      scok := (piece(x, u, 1) <> '0');
    end;

  if scok then
    begin
      FID             := piece(x, u, 1);
      FName           := piece(x, u, 2);
      FDefaultQryID   := piece(x, u, 3);
      FDefaultQryName := piece(x, u, 4);
      FUci            := piece(x, u, 5);
      FVolume         := piece(x, u, 6);
      FNode           := piece(x, u, 7);
      FBoxLookup      := piece(x, u, 8);
      FInstitution    := piece(x, u, 9);  {Added 1/31/97 - RV}
      LoadUser := True;
    end
  else
    begin
      Error.LoadAndDisplay(RPCBroker.Results);
      LoadUser := False;
    end;
end;

function TScUser.CheckQueryName(RPCBroker: TRPCBroker; QryName: TScStr50): TScStr20;
begin
  with RPCBroker do
    begin
      RemoteProcedure := 'SCRP QUERY CHECK NAME';
      Param[0].PType := literal;
      Param[0].Value := QryName;
      Param[1].PType := literal;
      Param[1].Value := FID;
      Call;
      CheckQueryName := Results[0];
    end;
end;

{       ----------  TScError Methods  ----------}

{Creates the error object instance.}
constructor TScError.Create;
begin
  inherited Create;
  FErrorList := TStringlist.Create;
end;

destructor TScError.Destroy;
begin
  FErrorList.Free;
  inherited Destroy;
end;

procedure TScError.LoadErrors(list: TStrings);
var
  x: String;
begin
    x := list[0];
    FErrorCnt  := piece(x, u, 2);
    FProcess   := piece(x, u, 3);
    FErrorList.Assign(list);
    FErrorList.Delete(0);
end;

procedure TScError.DisplayErrors;
begin
  Application.CreateForm(TfrmErrorRpt, frmErrorRpt);
  try
    frmErrorRpt.pnlProcess.Caption  := ' Process: ' + FProcess;
    frmErrorRpt.pnlErrorCnt.Caption := ' Errors: '  + FErrorCnt;
    frmErrorRpt.pnlLineCnt.Caption  := ' Lines: '   + IntToStr(FErrorList.Count);
    frmErrorRpt.memErrors.Lines     := FErrorList;
    frmErrorRpt.ShowModal;
  finally
    frmErrorRpt.Free;
  end;
end;

procedure TScError.LoadAndDisplay(list: TStrings);
begin
  LoadErrors(list);
  DisplayErrors;
end;


end.
