unit VerInfo;

interface

uses SysUtils, WinTypes, Dialogs;

type
    EVerInfoError = class(Exception);
    ENoVerInfoError = class(Exception);

    TVerInfoType = (viCompanyName, viComments,
                 viFileDescription, viFileVersion,
                 viInternalName, viLegalCopyright,
                 viLegalTrademarks, viOriginalFilename);

    TVerInfoRes = class
    private
           Handle: DWord;
           Size: Integer;
           RezBuffer: String;
           TransTable: PLongInt;
    public
          constructor Create(FileName: String);
          function GetVerNameString(VerKind: TVerInfoType): String;
    end;


implementation

uses Windows;

const
     SFInfo = '\StringFileInfo\';

     VerNameArray: array[viCompanyName..viOriginalFileName] of String[20] =
     ('\CompanyName', '\Comments', '\FileDescription', '\FileVersion',
      '\InternalName', '\LegalCopyright', '\LegalTrademarks',
      '\OriginalFileName');

     VerTranslation: PChar = '\VarFileInfo\Translation';

constructor TVerInfoRes.Create(FileName: String);
var
   SBSize: UInt;

begin
     Size := GetFileVersionInfoSize(PChar(FileName), Handle);
     if Size <= 0 then
        raise ENoVerInfoError.Create('No Version Info Available.');
     SetLength(RezBuffer, Size);
     if not GetFileVersionInfo(PChar(FileName), Handle,
            Size, PChar(RezBuffer)) then
            raise EVerInfoError.Create('Cannot obtain version info.');
     if not VerQueryValue(PChar(RezBuffer), VerTranslation,
            pointer(TransTable), SBSize) then
            raise EVerInfoError.Create('No Language info.');
end;

function TVerInfoRes.GetVerNameString(VerKind: TVerInfoType): String;
const
     FormatStr = '%s%.4x%.4x%s%s';
var
   P: PChar;
   S: UInt;

begin
     Result := Format(FormatStr,[sfInfo,LoWord(TransTable^),
            HiWord(TransTable^), VerNameArray[VerKind], #0]);
     if VerQueryValue(PChar(RezBuffer), @Result[1], Pointer(P), S)
     then Result := StrPas(P)
     else Result := '';
end;                                                     


end.
