unit Uprutil;

interface
 uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Grids, DBTables, DB, DBGrids, StdCtrls, Menus, Buttons,
  ExtCtrls, inifiles, typinfo, DDEMan, {dsgnintf,}
  LZExpand;

 procedure ScaleForm(form:TForm);
 procedure ScaleForm1(form:Tform);
 procedure AdjustFonts(form:TForm;dw:longint;nw:longint);
 procedure ReSizeForm(pc:Tcontrol;r:real;fntname:string);
 procedure ReSizeFormBySides(pc:Tcontrol;hr:double;wr:double;fntname:string);
 Function ComponentHasProperty(comp:Tcomponent;PropName:string):Boolean;
 Procedure ActivateDataSets(f:TForm);
 Procedure DeActivateDataSets(f:TForm);
 Function Piece(str,delim:string;p:integer):String;
 Procedure ClearGridStrings(grid:TObject;ResetRowCount:Boolean);
 Function Strip(str:string; const chr:char):string;
 Function StripWhite(str:string):string;
 Procedure ShowStatus(box:Tedit;txt:String);
 Function PathDress(path:string;withSlash:Boolean):string;
 Function DelimCount(lin:string;delim:string):Integer;
 var
  ScreenHeight:longint;
  ScreenWidth:longint;
  ScreenFont:String;
  hfont,holdfont:THandle;

 implementation

 Function DelimCount(lin:string;delim:string):Integer;
 var {this function counts the number of 'delim' in 'lin' - like 2 arg M Length}
  i,j:integer;
 begin
  i:=0;j:=1;
  result:=1;
  if lin='' then exit;
  while j>0 do
   begin
    j:=Pos(delim,lin);
    if j > 0 then
     Delete(lin,1,j);
    inc(i);
   end;
  result:=i;
 end;


Function PathDress(path:string;withSlash:Boolean):string;
begin {dress the path appropriately with or without '\' on end;}
 result:=path;
 if withSlash then
  if path[length(path)]='\' then
   exit
  else
   begin
    path:=path+'\';
    result:=path;
    exit;
   end;
 if path[length(path)]='\' then
  path:=copy(path,1,length(path)-1);
 result:=path;
end;

Procedure ShowStatus(box:Tedit;txt:String);
begin
 box.text:=txt;
 box.refresh;
end;

Function Strip(str:string; const chr:char):string;
begin {strip leading and trailing chr from string)}
 while (str[1]=chr) and (str<>'') do {strip leading}
  str:=copy(str,2,999);
 while (str[length(str)]=chr) and (Str<>'') do {strip Trailing}
  str:=copy(str,1,length(str)-1);
 result:=str;
end;

Function StripWhite(str:string):string;
var
 wsp:string;
begin {strip leading and trailing white space from string)}
 wsp:=' ' + #9 + #13 + #10 + #12;
 while pos(str[1],wsp)>0 do {strip leading}
  str:=copy(str,2,999);
 while pos(str[length(str)],wsp)>0 do {strip Trailing}
  str:=copy(str,1,length(str)-1);
 result:=str;
end;

procedure ClearGridStrings(grid:TObject;ResetRowCount:Boolean);
var {Don't think this get rid of any associated objects so watch out}
 i:integer;
begin
 with grid as TStringGrid do
  begin
   for i:= 0 to pred(rowcount) do
    begin
     if succ(i) > fixedrows then rows[i].clear;
     rowheights[i]:=defaultrowheight;{takes care of my row disappear trick}
    end;
   {Set grid to have one non-fixed row}
   if ResetRowCount then RowCount := FixedRows + 1;
  end;
end;

Function Piece(str,delim:string;p:integer):String;
var
 i:integer;
begin
 Result:=str;
 If Str='' then exit;
 if delim='' then exit;
 str:= Str + delim;
 for i := 1 to p do
  begin
   if str='' then
   begin
    result:='';
    exit;
   end;
   Result:=copy(str,1,pos(delim,str)-1);
   Delete(str,1,pos(delim,str)+ length(delim)-1);
  end;
end;

Procedure ActivateDataSets(f:TForm);
var
 i:integer;
begin
 for i:=0 to pred(f.componentcount) do
  begin
   if (f.components[i] is TTable) then
    TTable(f.components[i]).active:=true;
   if (f.components[i] is TQuery) then
    TTable(f.components[i]).active:=true;
  end;
 end;

Procedure DeActivateDataSets(f:TForm);
var
 i:integer;
begin
 for i:=0 to pred(f.componentcount) do
  begin
   if (f.components[i] is TTable) then
    TTable(f.components[i]).cancel;
   if (f.components[i] is TQuery) then
    TTable(f.components[i]).cancel;
  end;
 end;

procedure ReSizeForm(pc:Tcontrol;r:real;fntname:string);
var
 i,j,k: integer;
 c: TControl;
begin
 if r=0 then r:=1; {prevent division by 0}
 if fntname='' then fntname:='Arial';
 hFont := CreateFont(round((longint(TForm(pc).font.size)+2 *r)), 0, 0, 0,
                     FW_BOLD, 0, 0, 0, 1, OUT_DEVICE_PRECIS, 0,
                     PROOF_QUALITY, FF_DONTCARE, 'fntname');
 hOldFont := TForm(pc).Font.Handle;
 TForm(pc).Font.Handle := hFont;
 DeleteObject(hOldFont);

 with pc do
  begin
   for i:=0 to ComponentCount - 1 do begin
    if Components[i] is TControl then begin
      c := TControl(Components[i]);
      if c is TCustomgrid then {TCustom Grid makes sure I get all grid types}
       begin
         for j:=0 to tstringgrid(c).rowcount-1 do
         tstringgrid(c).rowheights[j]:=round(longint(tstringgrid(c).rowheights[j]) * r);
         for k:=0 to tstringgrid(c).colcount-1 do
          tstringgrid(c).colwidths[k]:=round(longint(tstringgrid(c).colwidths[k]) * r);
        end;
      {Apparently recursion not required }
      if c.componentcount>0 then
       ReSizeForm(c,r,fntname);
      c.Left   := round(r * Longint(C.left));
      c.Top    := round(r * Longint(c.top));
      c.Width  := round(r * Longint(c.width)) ;
      c.Height := round(r * Longint(c.height));
      end;
  end;
 end;
 TForm(pc).clientheight:=round(longint(TForm(pc).height) * r);
 TForm(pc).clientwidth:=round(longint(TForm(pc).width) * r);

end;

procedure ReSizeFormBySides(pc:Tcontrol;hr:double;wr:double;fntname:string);
var
 {Make Controls fit new form Size}
 i,j,k,gh,gw,fh: integer;

 c: TControl;
 {pc - form or control, fntname - fontname}
 {hr - height ratio, wr width ratio}
begin
 if hr=0 then hr:=1; {prevent division by 0}
 if wr=0 then wr:=1; {prevent division by 0}
 if fntname='' then fntname:='Arial';
 if (hr > 0) then {get rid of this test soon}
  begin
   fh:= abs(round(longint(TForm(pc).font.height) * hr));
   hFont := CreateFont(fh, 0, 0, 0,
                     FW_BOLD, 0, 0, 0, 1, OUT_DEVICE_PRECIS, 0,
                     PROOF_QUALITY, FF_DONTCARE, Pchar(fntname));
   hOldFont := TForm(pc).Font.Handle;
   TForm(pc).Font.Handle := hFont;
   DeleteObject(hOldFont);
  end;
 with pc do
  begin
   for i:=0 to ComponentCount - 1 do begin
    if Components[i] is TControl then begin
      c := TControl(Components[i]);
      if c is TCustomgrid then {TCustom Grid makes sure I get all grid types}
       begin
        gw:=tstringGrid(c).width;
        gh:=tstringGrid(c).height;
        tstringGrid(c).setbounds(Round(longint(TstringGrid(c).left) * wr),
                                 Round(longint(TstringGrid(c).top) * hr),
                                 Round(longint(TstringGrid(c).width)* wr),
                                 Round(longint(TstringGrid(c).height)* hr));
        for j:=0 to tstringgrid(c).rowcount-1 do {adjust row heights}
         tstringgrid(c).rowheights[j]:=Trunc(longint(tstringgrid(c).rowheights[j]) * hr);
        for k:=0 to tstringgrid(c).colcount-1 do {adjust col counts (adjust for grid draw rounding)}
         tstringgrid(c).colwidths[k]:=Trunc((longint(tstringgrid(c).colwidths[k])-1) * wr);
        {adjust last column to fit}
        k:=tstringgrid(c).colcount-1;
        if ssVertical = TstringGrid(c).ScrollBars then
         TstringGrid(c).colwidths[k]:=TstringGrid(c).colwidths[k]-10 {for scroll bar}
        end
      else
       c.setbounds(round(wr * Longint(C.left)), round(hr * Longint(c.top)),
                   round(wr * Longint(c.width)),round(hr * Longint(c.height)));
      if c is TGroupBox then
       TGroupBox(c).font.size:=Round(TGroupBox(c).font.size * wr);
      if c.componentcount>0 then
        ReSizeFormBySides(c,hr,wr,fntname);
      end;
  end;
 end;
end;

procedure ScaleForm(form:Tform);
begin
 { see Delphi ti2861 Aug 95}
 if ScreenFont='' then ScreenFont:='Ariel';
 form.scaled:=false;
 form.font.name:=ScreenFont;
 form.autoscroll:=false;
 form.position:=poScreenCenter;
 if (screenheight<>0) and (screenwidth<>0) then
  begin
   form.scaled:=true;
   if (screen.width) <> screenwidth then
    begin
     form.height:=longint(form.height) * longint(screen.height) div screenheight;
     form.width:=longint(screenwidth) * longint(screen.width) div screenwidth;
     form.scaleby(screen.width,screenwidth);
     form.font.name:='Arial';
     form.autoscroll:=false;
     form.position:=poScreenCenter;
    end;
   end;
   AdjustFonts(form,screenwidth,form.width);
  end;

procedure ScaleForm1(form:Tform);
var
 x,y:LongInt;
begin
 { see Delphi ti2861 Aug 95}
 x:=0;
 if ScreenFont='' then ScreenFont:='Ariel';
 form.scaled:=false;
 form.font.name:=ScreenFont;
 form.position:=poScreenCenter;
 form.autoscroll:=false;
 if (screenheight<>0) and (screenwidth<>0) then
  begin
   form.scaled:=true;
   x:=getsystemmetrics(SM_CXSCREEN);
   y:=getSystemMetrics(SM_CYSCREEN);
   if (x<>ScreenHeight) or (y<>ScreenWidth) then
    begin
     form.height:=longint(form.height) * x div ScreenHeight;
     form.width:=longint(screenwidth) * y div ScreenWidth;
     form.scaleby(x,ScreenHeight);
     form.font.name:='Arial';
     form.autoscroll:=false;
     form.position:=poScreenCenter;
    end;
   end;
   AdjustFonts(form,x,form.width);
  end;


 procedure AdjustFonts(form:TForm;dw:Longint;nw:longint);
 var
  i:integer;
 begin
  with form do
   begin
    for i:=componentcount-1 downto 0 do
     begin
      if GetPropinfo(Classinfo,'font') <> nil then
       begin
        font.name:=ScreenFont;
        if (dw>0) and (nw>0) then font.size:=(dw div nw) * font.size;
       end;
     end;
    end;
   end;

Function ComponentHasProperty(comp:Tcomponent;PropName:string):Boolean;
 var
  PropType: PTypeInfo;
  PropInfo: PPropInfo;
 begin
  Propinfo:=nil;
  PropInfo := GetPropInfo(comp.ClassInfo, PropName);
  result:=Propinfo <> nil;
 end;

end.


